# -*- coding: utf-8 -*-
"""The pinfo CLI tool."""

import argparse
import collections
import logging
import os
import uuid

from plaso.cli import tool_options
from plaso.cli import tools
from plaso.cli import views
from plaso.cli.helpers import manager as helpers_manager
from plaso.engine import knowledge_base
from plaso.lib import definitions
from plaso.lib import errors
from plaso.lib import timelib
from plaso.serializer import json_serializer
from plaso.storage import factory as storage_factory


class PinfoTool(
    tools.CLITool,
    tool_options.StorageFileOptions):
  """Pinfo CLI tool."""

  NAME = u'pinfo'
  DESCRIPTION = (
      u'Shows information about a Plaso storage file, for example how it was '
      u'collected, what information was extracted from a source, etc.')

  _INDENTATION_LEVEL = 8

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(PinfoTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._compare_storage_file_path = None
    self._output_filename = None
    self._output_format = None
    self._storage_file_path = None

    self._verbose = False
    self.compare_storage_information = False

  def _CalculateStorageCounters(self, storage):
    """Calculates the counters of the entire storage.

    Args:
      storage (BaseStorage): storage.

    Returns:
      dict[str,collections.Counter]: storage counters.
    """
    analysis_reports_counter = collections.Counter()
    analysis_reports_counter_error = False
    event_labels_counter = collections.Counter()
    event_labels_counter_error = False
    parsers_counter = collections.Counter()
    parsers_counter_error = False

    for session in storage.GetSessions():
      # Check for a dict for backwards compatibility.
      if isinstance(session.analysis_reports_counter, dict):
        analysis_reports_counter += collections.Counter(
            session.analysis_reports_counter)
      elif isinstance(session.analysis_reports_counter, collections.Counter):
        analysis_reports_counter += session.analysis_reports_counter
      else:
        analysis_reports_counter_error = True

      # Check for a dict for backwards compatibility.
      if isinstance(session.event_labels_counter, dict):
        event_labels_counter += collections.Counter(
            session.event_labels_counter)
      elif isinstance(session.event_labels_counter, collections.Counter):
        event_labels_counter += session.event_labels_counter
      else:
        event_labels_counter_error = True

      # Check for a dict for backwards compatibility.
      if isinstance(session.parsers_counter, dict):
        parsers_counter += collections.Counter(session.parsers_counter)
      elif isinstance(session.parsers_counter, collections.Counter):
        parsers_counter += session.parsers_counter
      else:
        parsers_counter_error = True

    storage_counters = {}

    if not analysis_reports_counter_error:
      storage_counters[u'analysis_reports'] = analysis_reports_counter

    if not event_labels_counter_error:
      storage_counters[u'event_labels'] = event_labels_counter

    if not parsers_counter_error:
      storage_counters[u'parsers'] = parsers_counter

    return storage_counters

  def _CompareStorages(self, storage, compare_storage):
    """Compares the contents of two storages.

    Args:
      storage (BaseStorage): storage.
      compare_storage (BaseStorage): storage to compare against.

    Returns:
      bool: True if the content of the storages is identical.
    """
    storage_counters = self._CalculateStorageCounters(storage)
    compare_storage_counters = self._CalculateStorageCounters(compare_storage)

    # TODO: improve comparision, currently only total numbers are compared.

    return storage_counters == compare_storage_counters

  def _PrintAnalysisReportCounter(
      self, analysis_reports_counter, session_identifier=None):
    """Prints the analysis reports counter.

    Args:
      analysis_reports_counter (collections.Counter): number of analysis
          reports per analysis plugin.
      session_identifier (Optional[str]): session identifier.
    """
    if not analysis_reports_counter:
      return

    title = u'Reports generated per plugin'
    if session_identifier:
      title = u'{0:s}: {1:s}'.format(title, session_identifier)

    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type,
        column_names=[u'Plugin name', u'Number of reports'], title=title)

    for key, value in sorted(analysis_reports_counter.items()):
      if key == u'total':
        continue
      table_view.AddRow([key, value])

    try:
      total = analysis_reports_counter[u'total']
    except KeyError:
      total = u'N/A'

    table_view.AddRow([u'Total', total])

    table_view.Write(self._output_writer)

  def _PrintAnalysisReportsDetails(self, storage):
    """Prints the details of the analysis reports.

    Args:
      storage (BaseStorage): storage.
    """
    if not storage.HasAnalysisReports():
      self._output_writer.Write(u'No analysis reports stored.\n\n')
      return

    for index, analysis_report in enumerate(storage.GetAnalysisReports()):
      title = u'Analysis report: {0:d}'.format(index)
      table_view = views.ViewsFactory.GetTableView(
          self._views_format_type, title=title)

      table_view.AddRow([u'String', analysis_report.GetString()])

      table_view.Write(self._output_writer)

  def _PrintErrorsDetails(self, storage):
    """Prints the details of the errors.

    Args:
      storage (BaseStorage): storage.
    """
    if not storage.HasErrors():
      self._output_writer.Write(u'No errors stored.\n\n')
      return

    for index, error in enumerate(storage.GetErrors()):
      title = u'Error: {0:d}'.format(index)
      table_view = views.ViewsFactory.GetTableView(
          self._views_format_type, title=title)

      table_view.AddRow([u'Message', error.message])
      table_view.AddRow([u'Parser chain', error.parser_chain])

      path_specification = error.path_spec.comparable
      for path_index, line in enumerate(path_specification.split(u'\n')):
        if not line:
          continue

        if path_index == 0:
          table_view.AddRow([u'Path specification', line])
        else:
          table_view.AddRow([u'', line])

      table_view.Write(self._output_writer)

  def _PrintEventLabelsCounter(
      self, event_labels_counter, session_identifier=None):
    """Prints the event labels counter.

    Args:
      event_labels_counter (collections.Counter): number of event tags per
          label.
      session_identifier (Optional[str]): session identifier.
    """
    if not event_labels_counter:
      return

    title = u'Event tags generated per label'
    if session_identifier:
      title = u'{0:s}: {1:s}'.format(title, session_identifier)

    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type,
        column_names=[u'Label', u'Number of event tags'], title=title)

    for key, value in sorted(event_labels_counter.items()):
      if key == u'total':
        continue
      table_view.AddRow([key, value])

    try:
      total = event_labels_counter[u'total']
    except KeyError:
      total = u'N/A'

    table_view.AddRow([u'Total', total])

    table_view.Write(self._output_writer)

  def _PrintParsersCounter(self, parsers_counter, session_identifier=None):
    """Prints the parsers counter

    Args:
      parsers_counter (collections.Counter): number of events per parser or
          parser plugin.
      session_identifier (Optional[str]): session identifier.
    """
    if not parsers_counter:
      return

    title = u'Events generated per parser'
    if session_identifier:
      title = u'{0:s}: {1:s}'.format(title, session_identifier)

    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type,
        column_names=[u'Parser (plugin) name', u'Number of events'],
        title=title)

    for key, value in sorted(parsers_counter.items()):
      if key == u'total':
        continue
      table_view.AddRow([key, value])

    table_view.AddRow([u'Total', parsers_counter[u'total']])

    table_view.Write(self._output_writer)

  def _PrintPreprocessingInformation(self, storage, session_number=None):
    """Prints the details of the preprocessing information.

    Args:
      storage (BaseStorage): storage.
      session_number (Optional[int]): session number.
    """
    knowledge_base_object = knowledge_base.KnowledgeBase()

    storage.ReadPreprocessingInformation(knowledge_base_object)

    # TODO: replace session_number by session_identifier.
    system_configuration = knowledge_base_object.GetSystemConfigurationArtifact(
        session_identifier=session_number)
    if not system_configuration:
      return

    title = u'System configuration'
    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type, title=title)

    hostname = u'N/A'
    if system_configuration.hostname:
      hostname = system_configuration.hostname.name

    operating_system = system_configuration.operating_system or u'N/A'
    operating_system_product = (
        system_configuration.operating_system_product or u'N/A')
    operating_system_version = (
        system_configuration.operating_system_version or u'N/A')
    code_page = system_configuration.code_page or u'N/A'
    keyboard_layout = system_configuration.keyboard_layout or u'N/A'
    time_zone = system_configuration.time_zone or u'N/A'

    table_view.AddRow([u'Hostname', hostname])
    table_view.AddRow([u'Operating system', operating_system])
    table_view.AddRow([u'Operating system product', operating_system_product])
    table_view.AddRow([u'Operating system version', operating_system_version])
    table_view.AddRow([u'Code page', code_page])
    table_view.AddRow([u'Keyboard layout', keyboard_layout])
    table_view.AddRow([u'Time zone', time_zone])

    table_view.Write(self._output_writer)

    title = u'User accounts'
    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type,
        column_names=[u'Username', u'User directory'], title=title)

    for user_account in system_configuration.user_accounts:
      table_view.AddRow([
          user_account.username, user_account.user_directory])

    table_view.Write(self._output_writer)

  def _PrintSessionsDetails(self, storage):
    """Prints the details of the sessions.

    Args:
      storage (BaseStorage): storage.
    """
    for session_number, session in enumerate(storage.GetSessions()):
      session_identifier = uuid.UUID(hex=session.identifier)

      start_time = u'N/A'
      if session.start_time is not None:
        start_time = timelib.Timestamp.CopyToIsoFormat(session.start_time)

      completion_time = u'N/A'
      if session.completion_time is not None:
        completion_time = timelib.Timestamp.CopyToIsoFormat(
            session.completion_time)

      enabled_parser_names = u'N/A'
      if session.enabled_parser_names:
        enabled_parser_names = u', '.join(sorted(session.enabled_parser_names))

      command_line_arguments = session.command_line_arguments or u'N/A'
      parser_filter_expression = session.parser_filter_expression or u'N/A'
      preferred_encoding = session.preferred_encoding or u'N/A'
      filter_file = session.filter_file or u'N/A'

      title = u'Session: {0!s}'.format(session_identifier)
      table_view = views.ViewsFactory.GetTableView(
          self._views_format_type, title=title)

      table_view.AddRow([u'Start time', start_time])
      table_view.AddRow([u'Completion time', completion_time])
      table_view.AddRow([u'Product name', session.product_name])
      table_view.AddRow([u'Product version', session.product_version])
      table_view.AddRow([u'Command line arguments', command_line_arguments])
      table_view.AddRow([u'Parser filter expression', parser_filter_expression])
      table_view.AddRow([u'Enabled parser and plugins', enabled_parser_names])
      table_view.AddRow([u'Preferred encoding', preferred_encoding])
      table_view.AddRow([u'Debug mode', session.debug_mode])
      table_view.AddRow([u'Filter file', filter_file])

      table_view.Write(self._output_writer)

      if self._verbose:
        self._PrintPreprocessingInformation(storage, session_number + 1)

        self._PrintParsersCounter(
            session.parsers_counter, session_identifier=session_identifier)

        self._PrintAnalysisReportCounter(
            session.analysis_reports_counter,
            session_identifier=session_identifier)

        self._PrintEventLabelsCounter(
            session.event_labels_counter,
            session_identifier=session_identifier)

  def _PrintSessionsOverview(self, storage):
    """Prints a sessions overview.

    Args:
      storage (BaseStorage): storage.
    """
    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type, title=u'Sessions')

    for session in storage.GetSessions():
      start_time = timelib.Timestamp.CopyToIsoFormat(
          session.start_time)
      session_identifier = uuid.UUID(hex=session.identifier)
      table_view.AddRow([str(session_identifier), start_time])

    table_view.Write(self._output_writer)

  def _PrintStorageInformationAsText(self, storage):
    """Prints information about the storage as human-readable text.

    Args:
      storage (BaseStorage): storage.
    """
    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type, title=u'Plaso Storage Information')
    table_view.AddRow([u'Filename', os.path.basename(self._storage_file_path)])
    table_view.AddRow([u'Format version', storage.format_version])
    table_view.AddRow([u'Serialization format', storage.serialization_format])
    table_view.Write(self._output_writer)

    if storage.storage_type == definitions.STORAGE_TYPE_SESSION:
      self._PrintSessionsOverview(storage)
      self._PrintSessionsDetails(storage)

      storage_counters = self._CalculateStorageCounters(storage)

      if u'parsers' not in storage_counters:
        self._output_writer.Write(
            u'Unable to determine number of events generated per parser.\n')
      else:
        self._PrintParsersCounter(storage_counters[u'parsers'])

      if u'analysis_reports' not in storage_counters:
        self._output_writer.Write(
            u'Unable to determine number of reports generated per plugin.\n')
      else:
        self._PrintAnalysisReportCounter(storage_counters[u'analysis_reports'])

      if u'event_labels' not in storage_counters:
        self._output_writer.Write(
            u'Unable to determine number of event tags generated per label.\n')
      else:
        self._PrintEventLabelsCounter(storage_counters[u'event_labels'])

      self._PrintErrorsDetails(storage)
      self._PrintAnalysisReportsDetails(storage)

    elif storage.storage_type == definitions.STORAGE_TYPE_TASK:
      self._PrintTasksInformation(storage)

  def _PrintStorageInformationAsJSON(self, storage):
    """Writes a summary of sessions as machine-readable JSON.

    Args:
      storage (BaseStorage): storage.
    """
    serializer = json_serializer.JSONAttributeContainerSerializer
    self._output_writer.Write(u'{')
    for index, session in enumerate(storage.GetSessions()):
      json_string = serializer.WriteSerialized(session)
      if index != 0:
        self._output_writer.Write(u',\n')
      self._output_writer.Write(u'"session_{0:s}": {1:s} '.format(
          session.identifier, json_string))
    self._output_writer.Write(u'}')

  def _PrintTasksInformation(self, storage):
    """Prints information about the tasks.

    Args:
      storage (BaseStorage): storage.
    """
    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type, title=u'Tasks')

    for task_start, _ in storage.GetSessions():
      start_time = timelib.Timestamp.CopyToIsoFormat(
          task_start.timestamp)
      task_identifier = uuid.UUID(hex=task_start.identifier)
      table_view.AddRow([str(task_identifier), start_time])

    table_view.Write(self._output_writer)

  def CompareStorages(self):
    """Compares the contents of two storages.

    Returns:
      bool: True if the content of the storages is identical.
    """
    storage_file = storage_factory.StorageFactory.CreateStorageFileForFile(
        self._storage_file_path)
    if not storage_file:
      logging.error(
          u'Format of storage file: {0:s} not supported'.format(
              self._storage_file_path))
      return

    try:
      storage_file.Open(path=self._storage_file_path, read_only=True)
    except IOError as exception:
      logging.error(
          u'Unable to open storage file: {0:s} with error: {1:s}'.format(
              self._storage_file_path, exception))
      return

    compare_storage_file = (
        storage_factory.StorageFactory.CreateStorageFileForFile(
            self._compare_storage_file_path))
    if not compare_storage_file:
      logging.error(
          u'Format of storage file: {0:s} not supported'.format(
              self._compare_storage_file_path))
      return

    try:
      compare_storage_file.Open(
          path=self._compare_storage_file_path, read_only=True)
    except IOError as exception:
      logging.error(
          u'Unable to open storage file: {0:s} with error: {1:s}'.format(
              self._compare_storage_file_path, exception))
      storage_file.Close()
      return

    try:
      result = self._CompareStorages(storage_file, compare_storage_file)

    finally:
      compare_storage_file.Close()
      storage_file.Close()

    if result:
      self._output_writer.Write(u'Storage files are identical.\n')
    else:
      self._output_writer.Write(u'Storage files are different.\n')

    return result

  def ParseArguments(self):
    """Parses the command line arguments.

    Returns:
      bool: True if the arguments were successfully parsed.
    """
    self._ConfigureLogging()

    argument_parser = argparse.ArgumentParser(
        description=self.DESCRIPTION, add_help=False,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    self.AddBasicOptions(argument_parser)

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        argument_parser, names=[u'storage_file'])

    argument_parser.add_argument(
        u'-v', u'--verbose', dest=u'verbose', action=u'store_true',
        default=False, help=u'Print verbose output.')

    argument_parser.add_argument(
        u'--compare', dest=u'compare_storage_file', type=str,
        action=u'store', default=u'', metavar=u'STORAGE_FILE', help=(
            u'The path of the storage file to compare against.'))

    argument_parser.add_argument(
        u'-w', u'--write', metavar=u'OUTPUTFILE', dest=u'write',
        help=u'Output filename.')

    argument_parser.add_argument(
        u'--output-format', dest=u'output_format', type=str,
        choices=[u'text', u'json'], action=u'store', default=u'text',
        metavar=u'FORMAT', help=u'Type of output to produce')

    try:
      options = argument_parser.parse_args()
    except UnicodeEncodeError:
      # If we get here we are attempting to print help in a non-Unicode
      # terminal.
      self._output_writer.Write(u'\n')
      self._output_writer.Write(argument_parser.format_help())
      return False

    try:
      self.ParseOptions(options)
    except errors.BadConfigOption as exception:
      self._output_writer.Write(u'ERROR: {0!s}\n'.format(exception))
      self._output_writer.Write(u'\n')
      self._output_writer.Write(argument_parser.format_usage())
      return False

    return True

  def ParseOptions(self, options):
    """Parses the options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    self._ParseInformationalOptions(options)

    if self._debug_mode:
      logging_level = logging.DEBUG
    elif self._quiet_mode:
      logging_level = logging.WARNING
    else:
      logging_level = logging.INFO

    self._ConfigureLogging(log_level=logging_level)

    self._verbose = getattr(options, u'verbose', False)

    self._output_filename = getattr(options, u'write', None)

    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=[u'storage_file'])

    # TODO: move check into _CheckStorageFile.
    if not self._storage_file_path:
      raise errors.BadConfigOption(u'Missing storage file option.')

    if not os.path.isfile(self._storage_file_path):
      raise errors.BadConfigOption(
          u'No such storage file: {0:s}.'.format(self._storage_file_path))

    compare_storage_file_path = self.ParseStringOption(
        options, u'compare_storage_file')
    if compare_storage_file_path:
      if not os.path.isfile(compare_storage_file_path):
        raise errors.BadConfigOption(
            u'No such storage file: {0:s}.'.format(compare_storage_file_path))

      self._compare_storage_file_path = compare_storage_file_path
      self.compare_storage_information = True

    self._output_format = self.ParseStringOption(options, u'output_format')

    if self._output_filename:
      if os.path.exists(self._output_filename):
        raise errors.BadConfigOption(
            u'Output file already exists: {0:s}.'.format(self._output_filename))
      output_file_object = open(self._output_filename, u'wb')
      self._output_writer = tools.FileObjectOutputWriter(output_file_object)

  def PrintStorageInformation(self):
    """Prints the storage information."""
    storage_file = storage_factory.StorageFactory.CreateStorageFileForFile(
        self._storage_file_path)
    if not storage_file:
      logging.error(
          u'Format of storage file: {0:s} not supported'.format(
              self._storage_file_path))
      return

    try:
      storage_file.Open(path=self._storage_file_path, read_only=True)
    except IOError as exception:
      logging.error(
          u'Unable to open storage file: {0:s} with error: {1:s}'.format(
              self._storage_file_path, exception))
      return

    try:
      if self._output_format == u'json':
        self._PrintStorageInformationAsJSON(storage_file)
      elif self._output_format == u'text':
        self._PrintStorageInformationAsText(storage_file)
    finally:
      storage_file.Close()



# -*- coding: utf-8 -*-
"""Tests for the SHA-1 hasher implementation."""

import unittest

from plaso.analyzers.hashers import sha1

from tests import test_lib as shared_test_lib
from tests.analyzers.hashers import test_lib


class SHA1Test(test_lib.HasherTestCase):
  """Tests the SHA-1 hasher."""

  @shared_test_lib.skipUnlessHasTestFile([u'empty_file'])
  def testFileHashMatchesEmptyFile(self):
    """Tests that hasher matches the hash of an empty file."""
    expected_sha1 = u'da39a3ee5e6b4b0d3255bfef95601890afd80709'

    hasher = sha1.SHA1Hasher()
    self._AssertTestPathStringDigestMatch(
        hasher, [u'empty_file'], expected_sha1)

    hasher = sha1.SHA1Hasher()
    self._AssertTestPathBinaryDigestMatch(
        hasher, [u'empty_file'], expected_sha1.decode(u'hex'))

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testFileHashMatchesKnownFile(self):
    """Tests that hasher matches the hash of an empty file."""
    expected_sha1 = u'd9f264323004fd9518c0474967f80421e60e9813'

    hasher = sha1.SHA1Hasher()
    self._AssertTestPathStringDigestMatch(hasher, [u'ímynd.dd'], expected_sha1)

    hasher = sha1.SHA1Hasher()
    self._AssertTestPathBinaryDigestMatch(
        hasher, [u'ímynd.dd'], expected_sha1.decode(u'hex'))


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the install history plist plugin."""

import unittest

from plaso.formatters import plist  # pylint: disable=unused-import
from plaso.parsers.plist_plugins import install_history

from tests import test_lib as shared_test_lib
from tests.parsers.plist_plugins import test_lib


class InstallHistoryPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the install history plist plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'InstallHistory.plist'])
  def testProcess(self):
    """Tests the Process function."""
    plist_name = u'InstallHistory.plist'

    plugin = install_history.InstallHistoryPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    self.assertEqual(storage_writer.number_of_events, 7)

    # The order in which PlistParser generates events is nondeterministic
    # hence we sort the events.
    events = list(storage_writer.GetSortedEvents())

    expected_timestamps = [
        1384225175000000, 1388205491000000, 1388232883000000, 1388232883000000,
        1388232883000000, 1388232883000000, 1390941528000000]
    timestamps = sorted([event.timestamp for event in events])

    self.assertEqual(timestamps, expected_timestamps)

    event = events[0]
    self.assertEqual(event.key, u'')
    self.assertEqual(event.root, u'/item')

    expected_description = (
        u'Installation of [OS X 10.9 (13A603)] using [OS X Installer]. '
        u'Packages: com.apple.pkg.BaseSystemBinaries, '
        u'com.apple.pkg.BaseSystemResources, '
        u'com.apple.pkg.Essentials, com.apple.pkg.BSD, '
        u'com.apple.pkg.JavaTools, com.apple.pkg.AdditionalEssentials, '
        u'com.apple.pkg.AdditionalSpeechVoices, '
        u'com.apple.pkg.AsianLanguagesSupport, com.apple.pkg.MediaFiles, '
        u'com.apple.pkg.JavaEssentials, com.apple.pkg.OxfordDictionaries, '
        u'com.apple.pkg.X11redirect, com.apple.pkg.OSInstall, '
        u'com.apple.pkg.update.compatibility.2013.001.')
    self.assertEqual(event.desc, expected_description)

    expected_message = u'/item/ {0:s}'.format(expected_description)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])
    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The status view."""

import sys

try:
  import win32api
  import win32console
except ImportError:
  win32console = None

from dfvfs.lib import definitions as dfvfs_definitions

import plaso

from plaso.lib import definitions
from plaso.cli import tools


class StatusView(object):
  """Processing status view."""

  MODE_LINEAR = u'linear'
  MODE_WINDOW = u'window'

  _SOURCE_TYPES = {
      dfvfs_definitions.SOURCE_TYPE_DIRECTORY: u'directory',
      dfvfs_definitions.SOURCE_TYPE_FILE: u'single file',
      dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_DEVICE: (
          u'storage media device'),
      dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_IMAGE: (
          u'storage media image')}

  _UNITS_1024 = [u'B', u'KiB', u'MiB', u'GiB', u'TiB', u'EiB', u'ZiB', u'YiB']

  def __init__(self, output_writer, tool_name):
    """Initializes a status view.

    Args:
      output_writer (OutputWriter): output writer.
      tool_name (str): namd of the tool.
    """
    super(StatusView, self).__init__()
    self._filter_file = None
    self._mode = self.MODE_WINDOW
    self._output_writer = output_writer
    self._source_path = None
    self._source_type = None
    self._stdout_output_writer = isinstance(
        output_writer, tools.StdoutOutputWriter)
    self._storage_file_path = None
    self._tool_name = tool_name

  def _ClearScreen(self):
    """Clears the terminal/console screen."""
    if not win32console:
      # ANSI escape sequence to clear screen.
      self._output_writer.Write(b'\033[2J')
      # ANSI escape sequence to move cursor to top left.
      self._output_writer.Write(b'\033[H')

    else:
      # Windows cmd.exe does not support ANSI escape codes, thus instead we
      # fill the console screen buffer with spaces.
      top_left_coordinate = win32console.PyCOORDType(0, 0)
      screen_buffer = win32console.GetStdHandle(win32api.STD_OUTPUT_HANDLE)
      screen_buffer_information = screen_buffer.GetConsoleScreenBufferInfo()

      screen_buffer_attributes = screen_buffer_information[u'Attributes']
      screen_buffer_size = screen_buffer_information[u'Size']
      console_size = screen_buffer_size.X * screen_buffer_size.Y

      screen_buffer.FillConsoleOutputCharacter(
          u' ', console_size, top_left_coordinate)
      screen_buffer.FillConsoleOutputAttribute(
          screen_buffer_attributes, console_size, top_left_coordinate)
      screen_buffer.SetConsoleCursorPosition(top_left_coordinate)

  def _FormatAnalysisStatusTableRow(self, process_status):
    """Formats an analysis status table row.

    Args:
      process_status (ProcessStatus): processing status.

    Returns:
      str: processing status formatted as a row.
    """
    pid = u'{0:d}'.format(process_status.pid)

    used_memory = self._FormatSizeInUnitsOf1024(process_status.used_memory)

    events = u''
    if (process_status.number_of_consumed_events is not None and
        process_status.number_of_consumed_events_delta is not None):
      events = u'{0:d} ({1:d})'.format(
          process_status.number_of_consumed_events,
          process_status.number_of_consumed_events_delta)

    event_tags = u''
    if (process_status.number_of_produced_event_tags is not None and
        process_status.number_of_produced_event_tags_delta is not None):
      event_tags = u'{0:d} ({1:d})'.format(
          process_status.number_of_produced_event_tags,
          process_status.number_of_produced_event_tags_delta)

    reports = u''
    if (process_status.number_of_produced_reports is not None and
        process_status.number_of_produced_reports_delta is not None):
      reports = u'{0:d} ({1:d})'.format(
          process_status.number_of_produced_reports,
          process_status.number_of_produced_reports_delta)

    # The columns are 8-spaces aligned.
    return u''.join([
        process_status.identifier,
        u' ' * (24 - len(process_status.identifier)),
        pid,
        u' ' * (8 - len(pid)),
        process_status.status,
        u' ' * (16 - len(process_status.status)),
        used_memory,
        u' ' * (16 - len(used_memory)),
        events,
        u' ' * (16 - len(events)),
        event_tags,
        u' ' * (16 - len(event_tags)),
        reports])

  def _FormatExtractionStatusTableRow(self, process_status):
    """Formats an extraction status table row.

    Args:
      process_status (ProcessStatus): processing status.

    Returns:
      str: processing status formatted as a row.
    """
    pid = u'{0:d}'.format(process_status.pid)

    used_memory = self._FormatSizeInUnitsOf1024(process_status.used_memory)

    sources = u''
    if (process_status.number_of_produced_sources is not None and
        process_status.number_of_produced_sources_delta is not None):
      sources = u'{0:d} ({1:d})'.format(
          process_status.number_of_produced_sources,
          process_status.number_of_produced_sources_delta)

    events = u''
    if (process_status.number_of_produced_events is not None and
        process_status.number_of_produced_events_delta is not None):
      events = u'{0:d} ({1:d})'.format(
          process_status.number_of_produced_events,
          process_status.number_of_produced_events_delta)

    # TODO: shorten display name to fit in 80 chars and show the filename.

    # The columns are 8-spaces aligned.
    return u''.join([
        process_status.identifier,
        u' ' * (16 - len(process_status.identifier)),
        pid,
        u' ' * (8 - len(pid)),
        process_status.status,
        u' ' * (16 - len(process_status.status)),
        used_memory,
        u' ' * (16 - len(used_memory)),
        sources,
        u' ' * (16 - len(sources)),
        events,
        u' ' * (16 - len(events)),
        process_status.display_name])

  def _FormatSizeInUnitsOf1024(self, size):
    """Represents a number of bytes in units of 1024.

    Args:
      size (int): size in bytes.

    Returns:
      str: human readable string of the size.
    """
    magnitude_1024 = 0
    used_memory_1024 = float(size)
    while used_memory_1024 >= 1024:
      used_memory_1024 /= 1024
      magnitude_1024 += 1

    if magnitude_1024 > 0 and magnitude_1024 <= 7:
      return u'{0:.1f} {1:s}'.format(
          used_memory_1024, self._UNITS_1024[magnitude_1024])

    return u'{0:d} B'.format(size)

  def _PrintAnalysisStatusHeader(self):
    """Prints the analysis status header."""
    self._output_writer.Write(
        u'Storage file\t: {0:s}\n'.format(self._storage_file_path))

    self._output_writer.Write(u'\n')

  def _PrintAnalysisStatusUpdateLinear(self, processing_status):
    """Prints an analysis status update in linear mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    for worker_status in processing_status.workers_status:
      status_line = (
          u'{0:s} (PID: {1:d}) - events consumed: {2:d} - running: '
          u'{3!s}\n').format(
              worker_status.identifier, worker_status.pid,
              worker_status.number_of_consumed_events,
              worker_status.status not in definitions.PROCESSING_ERROR_STATUS)
      self._output_writer.Write(status_line)

  def _PrintAnalysisStatusUpdateWindow(self, processing_status):
    """Prints an analysis status update in window mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if self._stdout_output_writer:
      self._ClearScreen()

    output_text = u'plaso - {0:s} version {1:s}\n\n'.format(
        self._tool_name, plaso.__version__)
    self._output_writer.Write(output_text)

    self._PrintAnalysisStatusHeader()

    status_header = (
        u'Identifier              '
        u'PID     '
        u'Status          '
        u'Memory          '
        u'Events          '
        u'Tags            '
        u'Reports')
    if not win32console:
      # TODO: for win32console get current color and set intensity,
      # write the header separately then reset intensity.
      status_header = u'\x1b[1m{0:s}\x1b[0m'.format(status_header)

    status_table = [status_header]

    status_row = self._FormatAnalysisStatusTableRow(
        processing_status.foreman_status)
    status_table.append(status_row)

    for worker_status in processing_status.workers_status:
      status_row = self._FormatAnalysisStatusTableRow(worker_status)
      status_table.append(status_row)

    status_table.append(u'')
    self._output_writer.Write(u'\n'.join(status_table))
    self._output_writer.Write(u'\n')

    if processing_status.aborted:
      self._output_writer.Write(
          u'Processing aborted - waiting for clean up.\n\n')

    # TODO: remove update flicker. For win32console we could set the cursor
    # top left, write the table, clean the remainder of the screen buffer
    # and set the cursor at the end of the table.
    if self._stdout_output_writer:
      # We need to explicitly flush stdout to prevent partial status updates.
      sys.stdout.flush()

  def _PrintExtractionStatusUpdateLinear(self, processing_status):
    """Prints an extraction status update in linear mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    for worker_status in processing_status.workers_status:
      status_line = (
          u'{0:s} (PID: {1:d}) - events produced: {2:d} - file: {3:s} '
          u'- running: {4!s}\n').format(
              worker_status.identifier, worker_status.pid,
              worker_status.number_of_produced_events,
              worker_status.display_name,
              worker_status.status not in definitions.PROCESSING_ERROR_STATUS)
      self._output_writer.Write(status_line)

  def _PrintExtractionStatusUpdateWindow(self, processing_status):
    """Prints an extraction status update in window mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if self._stdout_output_writer:
      self._ClearScreen()

    output_text = u'plaso - {0:s} version {1:s}\n\n'.format(
        self._tool_name, plaso.__version__)
    self._output_writer.Write(output_text)

    self.PrintExtractionStatusHeader(processing_status)

    # TODO: for win32console get current color and set intensity,
    # write the header separately then reset intensity.
    status_header = (
        u'Identifier      '
        u'PID     '
        u'Status          '
        u'Memory          '
        u'Sources         '
        u'Events          '
        u'File')
    if not win32console:
      status_header = u'\x1b[1m{0:s}\x1b[0m'.format(status_header)

    status_table = [status_header]

    status_row = self._FormatExtractionStatusTableRow(
        processing_status.foreman_status)
    status_table.append(status_row)

    for worker_status in processing_status.workers_status:
      status_row = self._FormatExtractionStatusTableRow(worker_status)
      status_table.append(status_row)

    status_table.append(u'')
    self._output_writer.Write(u'\n'.join(status_table))
    self._output_writer.Write(u'\n')

    if processing_status.aborted:
      self._output_writer.Write(
          u'Processing aborted - waiting for clean up.\n\n')

    # TODO: remove update flicker. For win32console we could set the cursor
    # top left, write the table, clean the remainder of the screen buffer
    # and set the cursor at the end of the table.
    if self._stdout_output_writer:
      # We need to explicitly flush stdout to prevent partial status updates.
      sys.stdout.flush()

  def GetAnalysisStatusUpdateCallback(self):
    """Retrieves the analysis status update callback function.

    Returns:
      function: status update callback function or None.
    """
    if self._mode == self.MODE_LINEAR:
      return self._PrintAnalysisStatusUpdateLinear
    elif self._mode == self.MODE_WINDOW:
      return self._PrintAnalysisStatusUpdateWindow

  def GetExtractionStatusUpdateCallback(self):
    """Retrieves the extraction status update callback function.

    Returns:
      function: status update callback function or None.
    """
    if self._mode == self.MODE_LINEAR:
      return self._PrintExtractionStatusUpdateLinear
    elif self._mode == self.MODE_WINDOW:
      return self._PrintExtractionStatusUpdateWindow

  # TODO: refactor to protected method.
  def PrintExtractionStatusHeader(self, processing_status):
    """Prints the extraction status header.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    self._output_writer.Write(
        u'Source path\t: {0:s}\n'.format(self._source_path))
    self._output_writer.Write(
        u'Source type\t: {0:s}\n'.format(self._source_type))

    if self._filter_file:
      self._output_writer.Write(u'Filter file\t: {0:s}\n'.format(
          self._filter_file))

    if processing_status and processing_status.tasks_status:
      tasks_status = processing_status.tasks_status

      self._output_writer.Write(u'\n')

      status_header = (
          u'Tasks:          '
          u'Queued  '
          u'Processing      '
          u'To merge        '
          u'Abandoned       '
          u'Total')

      if not win32console:
        status_header = u'\x1b[1m{0:s}\x1b[0m\n'.format(status_header)
      else:
        status_header = u'{0:s}\n'.format(status_header)

      self._output_writer.Write(status_header)

      number_of_queued_tasks = u'{0:d}'.format(
          tasks_status.number_of_queued_tasks)
      number_of_tasks_processing = u'{0:d}'.format(
          tasks_status.number_of_tasks_processing)
      number_of_tasks_pending_merge = u'{0:d}'.format(
          tasks_status.number_of_tasks_pending_merge)
      number_of_abandoned_tasks = u'{0:d}'.format(
          tasks_status.number_of_abandoned_tasks)
      total_number_of_tasks = u'{0:d}'.format(
          tasks_status.total_number_of_tasks)

      # The columns are 8-spaces aligned.
      status_line = u''.join([
          u' ' * 16,
          number_of_queued_tasks,
          u' ' * (8 - len(number_of_queued_tasks)),
          number_of_tasks_processing,
          u' ' * (16 - len(number_of_tasks_processing)),
          number_of_tasks_pending_merge,
          u' ' * (16 - len(number_of_tasks_pending_merge)),
          number_of_abandoned_tasks,
          u' ' * (16 - len(number_of_abandoned_tasks)),
          total_number_of_tasks])

      self._output_writer.Write(status_line)
      self._output_writer.Write(u'\n')

    self._output_writer.Write(u'\n')

  def PrintExtractionSummary(self, processing_status):
    """Prints a summary of the extraction.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if not processing_status:
      self._output_writer.Write(
          u'WARNING: missing processing status information.\n')

    elif not processing_status.aborted:
      if processing_status.error_path_specs:
        self._output_writer.Write(u'Processing completed with errors.\n')
      else:
        self._output_writer.Write(u'Processing completed.\n')

      number_of_errors = (
          processing_status.foreman_status.number_of_produced_errors)
      if number_of_errors:
        output_text = u'\n'.join([
            u'',
            (u'Number of errors encountered while extracting events: '
             u'{0:d}.').format(number_of_errors),
            u'',
            u'Use pinfo to inspect errors in more detail.',
            u''])
        self._output_writer.Write(output_text)

      if processing_status.error_path_specs:
        output_text = u'\n'.join([
            u'',
            u'Path specifications that could not be processed:',
            u''])
        self._output_writer.Write(output_text)
        for path_spec in processing_status.error_path_specs:
          self._output_writer.Write(path_spec.comparable)
          self._output_writer.Write(u'\n')

    self._output_writer.Write(u'\n')

  def SetMode(self, mode):
    """Sets the mode.

    Args:
      mode (str): status view mode.
    """
    self._mode = mode

  def SetSourceInformation(self, source_path, source_type, filter_file=None):
    """Sets the source information.

    Args:
      source_path (str): path of the source.
      source_type (str): source type.
      filter_file (Optional[str]): filter file.
    """
    self._filter_file = filter_file
    self._source_path = source_path
    self._source_type = self._SOURCE_TYPES.get(source_type, u'UNKNOWN')

  def SetStorageFileInformation(self, storage_file_path):
    """Sets the storage file information.

    Args:
      storage_file_path (str): path to the storage file.
    """
    self._storage_file_path = storage_file_path



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the psteal CLI tool."""

import os
import unittest

from dfvfs.resolver import resolver as dfvfs_resolver

from plaso.cli import psteal_tool
from plaso.lib import errors

from tests import test_lib as shared_test_lib
from tests.cli import test_lib


class PstealToolTest(test_lib.CLIToolTestCase):
  """Tests for the psteal CLI tool."""

  # pylint: disable=protected-access

  _BDE_PASSWORD = u'bde-TEST'

  _EXPECTED_PROCESSING_OPTIONS = u'\n'.join([
      u'usage: psteal_test.py',
      u'',
      u'Test argument parser.',
      u''])

  _STORAGE_FILENAME_TEMPLATE = r'\d{{8}}T\d{{6}}-{filename}.plaso'

  def testGenerateStorageFileName(self):
    """Tests the _GenerateStorageFileName function."""
    test_tool = psteal_tool.PstealTool()

    test_tool._source_path = u'/test/storage/path'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'path')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    test_tool._source_path = u'/test/storage/path/'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'path')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    test_tool._source_path = u'/'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'ROOT')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    test_tool._source_path = u'/foo/..'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'ROOT')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

    test_tool._source_path = u'foo/../bar'
    storage_filename = test_tool._GenerateStorageFileName()
    expected_storage_filename = self._STORAGE_FILENAME_TEMPLATE.format(
        filename=u'bar')
    self.assertRegexpMatches(storage_filename, expected_storage_filename)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath([u'artifacts'])

    # Test when the output file is missing.
    expected_error = (u'Output format: dynamic requires an output file')
    with self.assertRaisesRegexp(errors.BadConfigOption, expected_error):
      test_tool.ParseOptions(options)

    options.write = u'dynamic.out'

    # Test when the source is missing.
    expected_error = u'Missing source path.'
    with self.assertRaisesRegexp(errors.BadConfigOption, expected_error):
      test_tool.ParseOptions(options)

    with shared_test_lib.TempDirectory() as temp_directory:
      options.source = self._GetTestFilePath([u'testdir'])
      options.write = os.path.join(temp_directory, u'dynamic.out')

      # Test when both source and output are specified.
      test_tool.ParseOptions(options)

      with open(options.write, 'w') as file_object:
        file_object.write(u'bogus')

      # Test when output file already exists.
      # Escape \ otherwise assertRaisesRegexp can error with:
      # error: bogus escape: u'\\1'
      expected_error = u'Output file already exists: {0:s}.'.format(
          options.write.replace(u'\\', u'\\\\'))
      with self.assertRaisesRegexp(errors.BadConfigOption, expected_error):
        test_tool.ParseOptions(options)

  def testParseArguments(self):
    """Tests the ParseArguments function"""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    # Test ParseArguments with no output file nor source.
    result = test_tool.ParseArguments()
    self.assertFalse(result)
    output = output_writer.ReadOutput()
    expected_error = u'ERROR: Output format: dynamic requires an output file'
    self.assertIn(expected_error, output)

  def testExtractEventsFromSourceDirectory(self):
    """Tests the ExtractEventsFromSources function on a directory."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath([u'artifacts'])
    options.quiet = True
    options.status_view_mode = u'none'
    options.source = self._GetTestFilePath([u'testdir'])

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, u'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, u'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode(u'utf-8')),
          b'Source type\t: directory',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self.assertEqual(output.split(b'\n'), expected_output)

  def testExtractEventsFromSourceBDEImage(self):
    """Tests the ExtractEventsFromSources function on an image with BDE."""
    # TODO: added for testing.
    dfvfs_resolver.Resolver.key_chain.Empty()

    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath([u'artifacts'])
    options.credentials = [u'password:{0:s}'.format(self._BDE_PASSWORD)]
    options.quiet = True
    options.source = self._GetTestFilePath([u'bdetogo.raw'])
    options.status_view_mode = u'none'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, u'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, u'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode(u'utf-8')),
          b'Source type\t: storage media image',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self.assertEqual(output.split(b'\n'), expected_output)

  def testExtractEventsFromSourcesImage(self):
    """Tests the ExtractEventsFromSources function on a single partition."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath([u'artifacts'])
    options.quiet = True
    options.status_view_mode = u'none'
    options.source = self._GetTestFilePath([u'ímynd.dd'])

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, u'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, u'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode(u'utf-8')),
          b'Source type\t: storage media image',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self.assertEqual(output.split(b'\n'), expected_output)

  def testExtractEventsFromSourcePartitionedImage(self):
    """Tests the ExtractEventsFromSources function on a multi partition
    image."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath([u'artifacts'])
    options.partitions = u'all'
    options.quiet = True
    options.status_view_mode = u'none'
    # Note that the source file is a RAW (VMDK flat) image.
    options.source = self._GetTestFilePath([u'multi_partition_image.vmdk'])

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, u'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, u'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode(u'utf-8')),
          b'Source type\t: storage media image',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self.assertEqual(output.split(b'\n'), expected_output)

  def testExtractEventsFromSourceVSSImage(self):
    """Tests the ExtractEventsFromSources function on an image with VSS."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath([u'artifacts'])
    options.quiet = True
    options.single_process = True
    options.status_view_mode = u'none'
    options.source = self._GetTestFilePath([u'vsstest.qcow2'])
    options.vss_stores = u'all'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, u'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, u'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode(u'utf-8')),
          b'Source type\t: storage media image',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'Number of errors encountered while extracting events: 1.',
          b'',
          b'Use pinfo to inspect errors in more detail.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self.assertEqual(output.split(b'\n'), expected_output)

  def testExtractEventsFromSourceSingleFile(self):
    """Tests the ExtractEventsFromSources function on a single file."""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath([u'artifacts'])
    options.quiet = True
    options.status_view_mode = u'none'
    options.source = self._GetTestFilePath([u'System.evtx'])

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, u'unused_output.txt')
      options.storage_file = os.path.join(temp_directory, u'storage.plaso')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          b'',
          b'Source path\t: {0:s}'.format(options.source.encode(u'utf-8')),
          b'Source type\t: single file',
          b'',
          b'Processing started.',
          b'Processing completed.',
          b'',
          b'']

      output = output_writer.ReadOutput()
      self.assertEqual(output.split(b'\n'), expected_output)

  def testProcessStorage(self):
    """Test the AnalyzeEvents function"""
    output_writer = test_lib.TestOutputWriter(encoding=u'utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = self._GetTestFilePath([u'artifacts'])
    options.storage_file = self._GetTestFilePath([u'psort_test.json.plaso'])
    options.source = u'unused_source'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.write = os.path.join(temp_directory, u'output.txt')

      test_tool.ParseOptions(options)
      test_tool.AnalyzeEvents()

      expected_output_file_name = self._GetTestFilePath(
          [u'end_to_end', u'dynamic.log'])
      with open(expected_output_file_name, 'rb') as file_object:
        expected_output = file_object.read()

      with open(options.write, 'rb') as file_object:
        result_output = file_object.read()

      expected_output = sorted(expected_output.split(b'\n'))
      result_output = sorted(result_output.split(b'\n'))
      self.assertEqual(expected_output, result_output)

    output = output_writer.ReadOutput()
    self.assertIn(u'Events processed : 38', output)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This file contains tests for the task manager."""

import time
import unittest

from dfvfs.lib import definitions as dfvfs_definitions

from plaso.containers import tasks
from plaso.multi_processing import task_manager

from tests import test_lib as shared_test_lib


class TaskManagerTestCase(shared_test_lib.BaseTestCase):
  """Tests for the TaskManager."""

  # pylint: disable=protected-access

  _TEST_SESSION_IDENTIFIER = u'4'

  def testCreateTask(self):
    """Tests the CreateTask methods."""
    manager = task_manager.TaskManager()
    task = manager.CreateTask(self._TEST_SESSION_IDENTIFIER)
    self.assertIsInstance(task, tasks.Task)
    self.assertEqual(task.session_identifier, self._TEST_SESSION_IDENTIFIER)
    self.assertIsNotNone(task.identifier)

  def testTaskProcessing(self):
    """Tests the UpdateTaskAsProcessing and GetTasksCheckMerge methods."""
    manager = task_manager.TaskManager()
    tasks_processing = manager.GetTasksCheckMerge()
    self.assertEqual(0, len(tasks_processing))

    task = manager.CreateTask(self._TEST_SESSION_IDENTIFIER)
    self.assertIsNone(task.last_processing_time)

    manager.UpdateTaskAsProcessingByIdentifier(task.identifier)
    tasks_processing = manager.GetTasksCheckMerge()
    self.assertEqual(1, len(tasks_processing))
    task = tasks_processing[0]
    self.assertIsNotNone(task.last_processing_time)

    # Simulate the foreman marking a task as complete while the manager
    # believed it to still be processing.
    manager.CompleteTask(task)

  def testPendingMerge(self):
    """Tests the UpdateTaskPendingMerge and GetTaskPending merge methods."""
    manager = task_manager.TaskManager()
    task_pending_merge = manager.GetTaskPendingMerge(None)
    self.assertIsNone(task_pending_merge)

    task = manager.CreateTask(self._TEST_SESSION_IDENTIFIER)
    with self.assertRaises(ValueError):
      manager.UpdateTaskAsPendingMerge(task)
    manager.UpdateTaskAsProcessingByIdentifier(task.identifier)

    task_pending_merge = manager.GetTaskPendingMerge(None)
    self.assertIsNone(task_pending_merge)

    task.storage_file_size = 10
    manager.UpdateTaskAsPendingMerge(task)
    task_pending_merge = manager.GetTaskPendingMerge(None)
    self.assertEqual(task, task_pending_merge)
    manager.CompleteTask(task)

    small_task = manager.CreateTask(self._TEST_SESSION_IDENTIFIER)
    small_task.storage_file_size = 100
    manager.UpdateTaskAsProcessingByIdentifier(small_task.identifier)
    large_task = manager.CreateTask(self._TEST_SESSION_IDENTIFIER)
    large_task.storage_file_size = 1000
    manager.UpdateTaskAsProcessingByIdentifier(large_task.identifier)
    directory_task = manager.CreateTask(self._TEST_SESSION_IDENTIFIER)
    directory_task.file_entry_type = dfvfs_definitions.FILE_ENTRY_TYPE_DIRECTORY
    directory_task.storage_file_size = 1000
    manager.UpdateTaskAsProcessingByIdentifier(directory_task.identifier)

    manager.UpdateTaskAsPendingMerge(small_task)
    # Simulate a delayed update from a worker that the task is still processing
    manager.UpdateTaskAsProcessingByIdentifier(small_task.identifier)
    manager.UpdateTaskAsPendingMerge(directory_task)
    # Test that a directory task preempts a small task.
    merging_task = manager.GetTaskPendingMerge(small_task)
    self.assertEqual(merging_task, directory_task)
    self.assertEqual(manager._tasks_merging.keys(), [directory_task.identifier])
    manager.CompleteTask(directory_task)

    # Test that a small task is not preempted by a large task.
    manager.UpdateTaskAsPendingMerge(large_task)
    merging_task = manager.GetTaskPendingMerge(small_task)
    self.assertEqual(merging_task, small_task)
    self.assertEqual(manager._tasks_merging.keys(), [small_task.identifier])

  def testTaskAbandonment(self):
    """Tests the abandoning and adoption of tasks"""
    manager = task_manager.TaskManager()
    task = manager.CreateTask(self._TEST_SESSION_IDENTIFIER)
    self.assertEqual(manager.GetAbandonedTasks(), [])
    self.assertTrue(manager.HasPendingTasks())
    self.assertIsNone(manager.GetRetryTask())

    manager.UpdateTaskAsProcessingByIdentifier(task.identifier)
    timestamp = int(time.time() * 1000000)
    inactive_time = timestamp - task_manager.TaskManager._TASK_INACTIVE_TIME
    task.last_processing_time = inactive_time - 1
    # HasPendingTasks is responsible for marking tasks as abandoned, and it
    # should still return True if the task is abandoned, as it needs to be
    # retried.
    self.assertTrue(manager.HasPendingTasks())
    abandoned_tasks = manager.GetAbandonedTasks()
    self.assertIn(task, abandoned_tasks)
    self.assertTrue(manager._TaskIsRetriable(task))

    retry_task = manager.GetRetryTask()
    self.assertIsNotNone(retry_task)
    self.assertEqual(task.identifier, retry_task.original_task_identifier)
    self.assertTrue(task.retried)
    self.assertFalse(manager._TaskIsRetriable(retry_task))
    manager.CompleteTask(retry_task)

    manager.UpdateTaskAsProcessingByIdentifier(task.identifier)
    self.assertEqual(manager.GetAbandonedTasks(), [])
    self.assertTrue(manager.HasPendingTasks())

  # TODO: Add tests for updating tasks.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The formatter mediator object."""

import os

from plaso.formatters import winevt_rc
from plaso.lib import py2to3
from plaso.winnt import language_ids


class FormatterMediator(object):
  """Class that implements the formatter mediator."""

  DEFAULT_LANGUAGE_IDENTIFIER = u'en-US'
  # TODO: add smarter language ID to LCID resolving e.g.
  # 'en-US' falls back to 'en'.
  # LCID 0x0409 is en-US.
  DEFAULT_LCID = 0x0409

  _WINEVT_RC_DATABASE = u'winevt-rc.db'

  def __init__(self, data_location=None):
    """Initializes a formatter mediator object.

    Args:
      data_location: the path of the formatter data files.
    """
    super(FormatterMediator, self).__init__()
    self._data_location = data_location
    self._language_identifier = self.DEFAULT_LANGUAGE_IDENTIFIER
    self._lcid = self.DEFAULT_LCID
    self._winevt_database_reader = None

  def _GetWinevtRcDatabaseReader(self):
    """Opens the Windows Event Log resource database reader.

    Returns:
      WinevtResourcesSqlite3DatabaseReader: Windows Event Log resource
          database reader or None.
    """
    if not self._winevt_database_reader and self._data_location:
      database_path = os.path.join(
          self._data_location, self._WINEVT_RC_DATABASE)
      if not os.path.isfile(database_path):
        return

      self._winevt_database_reader = (
          winevt_rc.WinevtResourcesSqlite3DatabaseReader())
      if not self._winevt_database_reader.Open(database_path):
        self._winevt_database_reader = None

    return self._winevt_database_reader

  @property
  def lcid(self):
    """int: preferred Language Code identifier (LCID)."""
    return self._lcid

  def GetWindowsEventMessage(self, log_source, message_identifier):
    """Retrieves the message string for a specific Windows Event Log source.

    Args:
      log_source (str): Event Log source, such as "Application Error".
      message_identifier (int): message identifier.

    Returns:
      str: message string or None if not available.
    """
    database_reader = self._GetWinevtRcDatabaseReader()
    if not database_reader:
      return

    if self._lcid != self.DEFAULT_LCID:
      message_string = database_reader.GetMessage(
          log_source, self.lcid, message_identifier)
      if message_string:
        return message_string

    return database_reader.GetMessage(
        log_source, self.DEFAULT_LCID, message_identifier)

  def SetPreferredLanguageIdentifier(self, language_identifier):
    """Sets the preferred language identifier.

    Args:
      language_identifier (str): language identifier string such as "en-US"
          for US English or "is-IS" for Icelandic.

    Raises:
      KeyError: if the language identifier is not defined.
      TypeError: if the language identifier is not a string type.
    """
    if not isinstance(language_identifier, py2to3.STRING_TYPES):
      raise ValueError(u'Language identifier is not a string.')

    values = language_ids.LANGUAGE_IDENTIFIERS.get(
        language_identifier.lower(), None)
    if not values:
      raise KeyError(u'Language identifier: {0:s} is not defined.'.format(
          language_identifier))
    self._language_identifier = language_identifier
    self._lcid = values[0]



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Twitter on iOS 8+ database event formatter."""

import unittest

from plaso.formatters import twitter_ios
from tests.formatters import test_lib


class TwitterIOSContactFormatterTest(test_lib.EventFormatterTestCase):
  """Tests the Twitter on iOS 8+ contact event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = twitter_ios.TwitterIOSContactFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = twitter_ios.TwitterIOSContactFormatter()

    expected_attribute_names = [
        u'screen_name',
        u'profile_url',
        u'name',
        u'location',
        u'description',
        u'url',
        u'following',
        u'followers_count',
        u'following_count',
    ]

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)


class TwitterIOSStatusFormatterTest(test_lib.EventFormatterTestCase):
  """Tests the Twitter on iOS 8+ status event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = twitter_ios.TwitterIOSStatusFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = twitter_ios.TwitterIOSStatusFormatter()

    expected_attribute_names = [
        u'text',
        u'user_id',
        u'name',
        u'retweet_count',
        u'favorite_count',
        u'favorited',
    ]

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the Windows NT time zone definitions.

The Windows time zone names can be obtained from the following
Windows Registry key:
HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones
"""

# Dictionary to map Windows time zone names to Python equivalents.
# Note that spaces have been stripped from the Windows time zone names.

TIME_ZONES = {
    u'AlaskanStandardTime': u'US/Alaska',
    u'Arabic Standard Time': u'Asia/Baghdad',
    u'AtlanticStandardTime': u'Canada/Atlantic',
    u'AzoresStandardTime': u'Atlantic/Azores',
    u'CanadaCentralStandardTime': u'CST6CDT',
    u'CapeVerdeStandardTime': u'Atlantic/Azores',
    u'CentralAmericaStandardTime': u'CST6CDT',
    u'CentralDaylightTime': u'CST6CDT',
    u'CentralEuropeanStandardTime': u'Europe/Belgrade',
    u'CentralEuropeStandardTime': u'Europe/Belgrade',
    u'Central Standard Time': u'CST6CDT',
    u'CentralStandardTime': u'CST6CDT',
    u'ChinaStandardTime': u'Asia/Bangkok',
    u'EasternDaylightTime': u'EST5EDT',
    u'EasternStandardTime': u'EST5EDT',
    u'E.EuropeStandardTime': u'Egypt',
    u'EgyptStandardTime': u'Egypt',
    u'GMTStandardTime': u'GMT',
    u'GreenwichStandardTime': u'GMT',
    u'HawaiianStandardTime': u'US/Hawaii',
    u'IndiaStandardTime': u'Asia/Kolkata',
    u'IsraelStandardTime': u'Egypt',
    u'KoreaStandardTime': u'Asia/Seoul',
    u'MalayPeninsulaStandardTime': u'Asia/Kuching',
    u'MexicoStandardTime2': u'MST7MDT',
    u'MexicoStandardTime': u'CST6CDT',
    u'MountainDaylightTime': u'MST7MDT',
    u'MountainStandardTime': u'MST7MDT',
    u'NewfoundlandStandardTime': u'Canada/Newfoundland',
    u'NorthAsiaEastStandardTime': u'Asia/Bangkok',
    u'PacificDaylightTime': u'PST8PDT',
    u'PacificSAStandardTime': u'Canada/Atlantic',
    u'Pacific Standard Time': u'PST8PDT',
    u'PacificStandardTime': u'PST8PDT',
    u'RomanceStandardTime': u'Europe/Belgrade',
    u'RomanceDaylightTime': u'Europe/Belgrade',
    u'SamoaStandardTime': u'US/Samoa',
    u'SAPacificStandardTime': u'EST5EDT',
    u'SAWesternStandardTime': u'Canada/Atlantic',
    u'SingaporeStandardTime': u'Asia/Bangkok',
    u'SouthAfricaStandardTime': u'Egypt',
    u'TaipeiStandardTime': u'Asia/Bangkok',
    u'TokyoStandardTime': u'Asia/Tokyo',
    u'@tzres.dll,-1010': u'Asia/Aqtau',
    u'@tzres.dll,-1020': u'Asia/Dhaka',
    u'@tzres.dll,-1021': u'Asia/Dhaka',
    u'@tzres.dll,-1022': u'Asia/Dhaka',
    u'@tzres.dll,-104': u'America/Cuiaba',
    u'@tzres.dll,-105': u'America/Cuiaba',
    u'@tzres.dll,-1070': u'Asia/Tbilisi',
    u'@tzres.dll,-10': u'Atlantic/Azores',
    u'@tzres.dll,-110': u'EST5EDT',
    u'@tzres.dll,-111': u'EST5EDT',
    u'@tzres.dll,-1120': u'America/Cuiaba',
    u'@tzres.dll,-112': u'EST5EDT',
    u'@tzres.dll,-1140': u'Pacific/Fiji',
    u'@tzres.dll,-11': u'Atlantic/Azores',
    u'@tzres.dll,-120': u'EST5EDT',
    u'@tzres.dll,-121': u'EST5EDT',
    u'@tzres.dll,-122': u'EST5EDT',
    u'@tzres.dll,-12': u'Atlantic/Azores',
    u'@tzres.dll,-130': u'EST5EDT',
    u'@tzres.dll,-131': u'EST5EDT',
    u'@tzres.dll,-132': u'EST5EDT',
    u'@tzres.dll,-140': u'CST6CDT',
    u'@tzres.dll,-141': u'CST6CDT',
    u'@tzres.dll,-142': u'CST6CDT',
    u'@tzres.dll,-1460': u'Pacific/Port_Moresby',
    u'@tzres.dll,-150': u'America/Guatemala',
    u'@tzres.dll,-151': u'America/Guatemala',
    u'@tzres.dll,-152': u'America/Guatemala',
    u'@tzres.dll,-1530': u'Asia/Yekaterinburg',
    u'@tzres.dll,-160': u'CST6CDT',
    u'@tzres.dll,-161': u'CST6CDT',
    u'@tzres.dll,-162': u'CST6CDT',
    u'@tzres.dll,-1630': u'Europe/Nicosia',
    u'@tzres.dll,-1660': u'America/Bahia',
    u'@tzres.dll,-1661': u'America/Bahia',
    u'@tzres.dll,-1662': u'America/Bahia',
    u'@tzres.dll,-170': u'America/Mexico_City',
    u'@tzres.dll,-171': u'America/Mexico_City',
    u'@tzres.dll,-172': u'America/Mexico_City',
    u'@tzres.dll,-180': u'MST7MDT',
    u'@tzres.dll,-181': u'MST7MDT',
    u'@tzres.dll,-182': u'MST7MDT',
    u'@tzres.dll,-190': u'MST7MDT',
    u'@tzres.dll,-191': u'MST7MDT',
    u'@tzres.dll,-192': u'MST7MDT',
    u'@tzres.dll,-200': u'MST7MDT',
    u'@tzres.dll,-201': u'MST7MDT',
    u'@tzres.dll,-202': u'MST7MDT',
    u'@tzres.dll,-20': u'Atlantic/Cape_Verde',
    u'@tzres.dll,-210': u'PST8PDT',
    u'@tzres.dll,-211': u'PST8PDT',
    u'@tzres.dll,-212': u'PST8PDT',
    u'@tzres.dll,-21': u'Atlantic/Cape_Verde',
    u'@tzres.dll,-220': u'US/Alaska',
    u'@tzres.dll,-221': u'US/Alaska',
    u'@tzres.dll,-222': u'US/Alaska',
    u'@tzres.dll,-22': u'Atlantic/Cape_Verde',
    u'@tzres.dll,-230': u'US/Hawaii',
    u'@tzres.dll,-231': u'US/Hawaii',
    u'@tzres.dll,-232': u'US/Hawaii',
    u'@tzres.dll,-260': u'GMT',
    u'@tzres.dll,-261': u'GMT',
    u'@tzres.dll,-262': u'GMT',
    u'@tzres.dll,-271': u'UTC',
    u'@tzres.dll,-272': u'UTC',
    u'@tzres.dll,-280': u'Europe/Budapest',
    u'@tzres.dll,-281': u'Europe/Budapest',
    u'@tzres.dll,-282': u'Europe/Budapest',
    u'@tzres.dll,-290': u'Europe/Warsaw',
    u'@tzres.dll,-291': u'Europe/Warsaw',
    u'@tzres.dll,-292': u'Europe/Warsaw',
    u'@tzres.dll,-300': u'Europe/Paris',
    u'@tzres.dll,-301': u'Europe/Paris',
    u'@tzres.dll,-302': u'Europe/Paris',
    u'@tzres.dll,-320': u'Europe/Berlin',
    u'@tzres.dll,-321': u'Europe/Berlin',
    u'@tzres.dll,-322': u'Europe/Berlin',
    u'@tzres.dll,-331': u'Europe/Nicosia',
    u'@tzres.dll,-332': u'Europe/Nicosia',
    u'@tzres.dll,-340': u'Africa/Cairo',
    u'@tzres.dll,-341': u'Africa/Cairo',
    u'@tzres.dll,-342': u'Africa/Cairo',
    u'@tzres.dll,-350': u'Europe/Sofia',
    u'@tzres.dll,-351': u'Europe/Sofia',
    u'@tzres.dll,-352': u'Europe/Sofia',
    u'@tzres.dll,-365': u'Egypt',
    u'@tzres.dll,-371': u'Asia/Jerusalem',
    u'@tzres.dll,-372': u'Asia/Jerusalem',
    u'@tzres.dll,-380': u'Africa/Harare',
    u'@tzres.dll,-381': u'Africa/Harare',
    u'@tzres.dll,-382': u'Africa/Harare',
    u'@tzres.dll,-390': u'Asia/Kuwait',
    u'@tzres.dll,-391': u'Asia/Kuwait',
    u'@tzres.dll,-392': u'Asia/Kuwait',
    u'@tzres.dll,-400': u'Asia/Baghdad',
    u'@tzres.dll,-401': u'Asia/Baghdad',
    u'@tzres.dll,-402': u'Asia/Baghdad',
    u'@tzres.dll,-40': u'Brazil/East',
    u'@tzres.dll,-410': u'Africa/Nairobi',
    u'@tzres.dll,-411': u'Africa/Nairobi',
    u'@tzres.dll,-412': u'Africa/Nairobi',
    u'@tzres.dll,-41': u'Brazil/East',
    u'@tzres.dll,-42': u'Brazil/East',
    u'@tzres.dll,-420': u'Europe/Moscow',
    u'@tzres.dll,-421': u'Europe/Moscow',
    u'@tzres.dll,-422': u'Europe/Moscow',
    u'@tzres.dll,-434': u'Asia/Tbilisi',
    u'@tzres.dll,-435': u'Asia/Tbilisi',
    u'@tzres.dll,-440': u'Asia/Muscat',
    u'@tzres.dll,-441': u'Asia/Muscat',
    u'@tzres.dll,-442': u'Asia/Muscat',
    u'@tzres.dll,-447': u'Asia/Baku',
    u'@tzres.dll,-448': u'Asia/Baku',
    u'@tzres.dll,-449': u'Asia/Baku',
    u'@tzres.dll,-450': u'Asia/Yerevan',
    u'@tzres.dll,-451': u'Asia/Yerevan',
    u'@tzres.dll,-452': u'Asia/Yerevan',
    u'@tzres.dll,-460': u'Asia/Kabul',
    u'@tzres.dll,-461': u'Asia/Kabul',
    u'@tzres.dll,-462': u'Asia/Kabul',
    u'@tzres.dll,-471': u'Asia/Yekaterinburg',
    u'@tzres.dll,-472': u'Asia/Yekaterinburg',
    u'@tzres.dll,-480': u'Asia/Karachi',
    u'@tzres.dll,-481': u'Asia/Karachi',
    u'@tzres.dll,-482': u'Asia/Karachi',
    u'@tzres.dll,-490': u'Asia/Kolkata',
    u'@tzres.dll,-491': u'Asia/Kolkata',
    u'@tzres.dll,-492': u'Asia/Kolkata',
    u'@tzres.dll,-500': u'Asia/Kathmandu',
    u'@tzres.dll,-501': u'Asia/Kathmandu',
    u'@tzres.dll,-502': u'Asia/Kathmandu',
    u'@tzres.dll,-510': u'Asia/Dhaka',
    u'@tzres.dll,-511': u'Asia/Aqtau',
    u'@tzres.dll,-512': u'Asia/Aqtau',
    u'@tzres.dll,-570': u'Asia/Chongqing',
    u'@tzres.dll,-571': u'Asia/Chongqing',
    u'@tzres.dll,-572': u'Asia/Chongqing',
    u'@tzres.dll,-60': u'America/Buenos_Aires',
    u'@tzres.dll,-611': u'Australia/Perth',
    u'@tzres.dll,-612': u'Australia/Perth',
    u'@tzres.dll,-621': u'Asia/Seoul',
    u'@tzres.dll,-622': u'Asia/Seoul',
    u'@tzres.dll,-631': u'Asia/Tokyo',
    u'@tzres.dll,-632': u'Asia/Tokyo',
    u'@tzres.dll,-650': u'Australia/Darwin',
    u'@tzres.dll,-651': u'Australia/Darwin',
    u'@tzres.dll,-652': u'Australia/Darwin',
    u'@tzres.dll,-660': u'Australia/Adelaide',
    u'@tzres.dll,-661': u'Australia/Adelaide',
    u'@tzres.dll,-662': u'Australia/Adelaide',
    u'@tzres.dll,-670': u'Australia/Sydney',
    u'@tzres.dll,-671': u'Australia/Sydney',
    u'@tzres.dll,-672': u'Australia/Sydney',
    u'@tzres.dll,-680': u'Australia/Brisbane',
    u'@tzres.dll,-681': u'Australia/Brisbane',
    u'@tzres.dll,-682': u'Australia/Brisbane',
    u'@tzres.dll,-691': u'Australia/Hobart',
    u'@tzres.dll,-692': u'Australia/Hobart',
    u'@tzres.dll,-70': u'Canada/Newfoundland',
    u'@tzres.dll,-71': u'Canada/Newfoundland',
    u'@tzres.dll,-721': u'Pacific/Port_Moresby',
    u'@tzres.dll,-722': u'Pacific/Port_Moresby',
    u'@tzres.dll,-72': u'Canada/Newfoundland',
    u'@tzres.dll,-731': u'Pacific/Fiji',
    u'@tzres.dll,-732': u'Pacific/Fiji',
    u'@tzres.dll,-740': u'Pacific/Auckland',
    u'@tzres.dll,-741': u'Pacific/Auckland',
    u'@tzres.dll,-742': u'Pacific/Auckland',
    u'@tzres.dll,-80': u'Canada/Atlantic',
    u'@tzres.dll,-81': u'Canada/Atlantic',
    u'@tzres.dll,-82': u'Canada/Atlantic',
    u'@tzres.dll,-840': u'America/Argentina/Buenos_Aires',
    u'@tzres.dll,-841': u'America/Argentina/Buenos_Aires',
    u'@tzres.dll,-842': u'America/Argentina/Buenos_Aires',
    u'@tzres.dll,-870': u'Asia/Karachi',
    u'@tzres.dll,-871': u'Asia/Karachi',
    u'@tzres.dll,-872': u'Asia/Karachi',
    u'@tzres.dll,-880': u'UTC',
    u'@tzres.dll,-930': u'UTC',
    u'@tzres.dll,-931': u'UTC',
    u'@tzres.dll,-932': u'UTC',
    u'USEasternStandardTime': u'EST5EDT',
    u'USMountainStandardTime': u'MST7MDT',
    u'W.AustraliaStandardTime': u'Australia/Perth',
    u'W.CentralAfricaStandardTime': u'Europe/Belgrade',
    u'W.EuropeStandardTime': u'Europe/Belgrade',
    u'(UTC) Coordinated Universal Time': u'UTC',
}

# Values of @tzres.dll,-[0-9]+
# 10	(UTC-01:00) Azores
# 11	Azores Daylight Time
# 12	Azores Standard Time
# 20	(UTC-01:00) Cape Verde Is.
# 21	Cape Verde Daylight Time
# 22	Cape Verde Standard Time
# 30	(UTC-02:00) Mid-Atlantic
# 31	Mid-Atlantic Daylight Time
# 32	Mid-Atlantic Standard Time
# 40	(UTC-03:00) Brasilia
# 41	E. South America Daylight Time
# 42	E. South America Standard Time
# 50	(UTC-03:00) Greenland
# 51	Greenland Daylight Time
# 52	Greenland Standard Time
# 60	(UTC-03:00) Buenos Aires, Georgetown
# 61	SA Eastern Daylight Time
# 62	SA Eastern Standard Time
# 70	(UTC-03:30) Newfoundland
# 71	Newfoundland Daylight Time
# 72	Newfoundland Standard Time
# 80	(UTC-04:00) Atlantic Time (Canada)
# 81	Atlantic Daylight Time
# 82	Atlantic Standard Time
# 90	(UTC-04:00) Santiago
# 91	Pacific SA Daylight Time
# 92	Pacific SA Standard Time
# 100	(UTC-04:00) Caracas, La Paz
# 101	SA Western Daylight Time
# 102	SA Western Standard Time
# 103	(UTC-04:00) Manaus
# 104	Central Brazilian Daylight Time
# 105	Central Brazilian Standard Time
# 110	(UTC-05:00) Eastern Time (US & Canada)
# 111	Eastern Daylight Time
# 112	Eastern Standard Time
# 120	(UTC-05:00) Bogota, Lima, Quito, Rio Branco
# 121	SA Pacific Daylight Time
# 122	SA Pacific Standard Time
# 130	(UTC-05:00) Indiana (East)
# 131	US Eastern Daylight Time
# 132	US Eastern Standard Time
# 140	(UTC-06:00) Saskatchewan
# 141	Canada Central Daylight Time
# 142	Canada Central Standard Time
# 150	(UTC-06:00) Central America
# 151	Central America Daylight Time
# 152	Central America Standard Time
# 160	(UTC-06:00) Central Time (US & Canada)
# 161	Central Daylight Time
# 162	Central Standard Time
# 170	(UTC-06:00) Guadalajara, Mexico City, Monterrey
# 171	Central Daylight Time (Mexico)
# 172	Central Standard Time (Mexico)
# 180	(UTC-07:00) Chihuahua, La Paz, Mazatlan
# 181	Mountain Daylight Time (Mexico)
# 182	Mountain Standard Time (Mexico)
# 190	(UTC-07:00) Mountain Time (US & Canada)
# 191	Mountain Daylight Time
# 192	Mountain Standard Time
# 200	(UTC-07:00) Arizona
# 201	US Mountain Daylight Time
# 202	US Mountain Standard Time
# 210	(UTC-08:00) Pacific Time (US & Canada)
# 211	Pacific Daylight Time
# 212	Pacific Standard Time
# 213	(UTC-08:00) Tijuana, Baja California
# 214	Pacific Daylight Time (Mexico)
# 215	Pacific Standard Time (Mexico)
# 220	(UTC-09:00) Alaska
# 221	Alaskan Daylight Time
# 222	Alaskan Standard Time
# 230	(UTC-10:00) Hawaii
# 231	Hawaiian Daylight Time
# 232	Hawaiian Standard Time
# 240	(UTC-11:00) Midway Island, Samoa
# 241	Samoa Daylight Time
# 242	Samoa Standard Time
# 250	(UTC-12:00) International Date Line West
# 251	Dateline Daylight Time
# 252	Dateline Standard Time
# 260	(UTC) Dublin, Edinburgh, Lisbon, London
# 261	GMT Daylight Time
# 262	GMT Standard Time
# 270	(UTC) Casablanca, Monrovia, Reykjavik
# 271	Greenwich Daylight Time
# 272	Greenwich Standard Time
# 280	(UTC+01:00) Belgrade, Bratislava, Budapest, Ljubljana, Prague
# 281	Central Europe Daylight Time
# 282	Central Europe Standard Time
# 290	(UTC+01:00) Sarajevo, Skopje, Warsaw, Zagreb
# 291	Central European Daylight Time
# 292	Central European Standard Time
# 300	(UTC+01:00) Brussels, Copenhagen, Madrid, Paris
# 301	Romance Daylight Time
# 302	Romance Standard Time
# 310	(UTC+01:00) West Central Africa
# 311	W. Central Africa Daylight Time
# 312	W. Central Africa Standard Time
# 320	(UTC+01:00) Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna
# 321	W. Europe Daylight Time
# 322	W. Europe Standard Time
# 330	(UTC+02:00) Minsk
# 331	E. Europe Daylight Time
# 332	E. Europe Standard Time
# 333	(UTC+02:00) Amman
# 334	Jordan Daylight Time
# 335	Jordan Standard Time
# 340	(UTC+02:00) Cairo
# 341	Egypt Daylight Time
# 342	Egypt Standard Time
# 350	(UTC+02:00) Helsinki, Kyiv, Riga, Sofia, Tallinn, Vilnius
# 351	FLE Daylight Time
# 352	FLE Standard Time
# 360	(UTC+02:00) Athens, Bucharest, Istanbul
# 361	GTB Daylight Time
# 362	GTB Standard Time
# 363	(UTC+02:00) Beirut
# 364	Middle East Daylight Time
# 365	Middle East Standard Time
# 370	(UTC+02:00) Jerusalem
# 371	Jerusalem Daylight Time
# 372	Jerusalem Standard Time
# 380	(UTC+02:00) Harare, Pretoria
# 381	South Africa Daylight Time
# 382	South Africa Standard Time
# 383	(UTC+02:00) Windhoek
# 384	Namibia Daylight Time
# 385	Namibia Standard Time
# 390	(UTC+03:00) Kuwait, Riyadh
# 391	Arab Daylight Time
# 392	Arab Standard Time
# 400	(UTC+03:00) Baghdad
# 401	Arabic Daylight Time
# 402	Arabic Standard Time
# 410	(UTC+03:00) Nairobi
# 411	E. Africa Daylight Time
# 412	E. Africa Standard Time
# 420	(UTC+03:00) Moscow, St. Petersburg, Volgograd
# 421	Russian Daylight Time
# 422	Russian Standard Time
# 430	(UTC+03:30) Tehran
# 431	Iran Daylight Time
# 432	Iran Standard Time
# 433	(UTC+03:00) Tbilisi
# 434	Georgian Daylight Time
# 435	Georgian Standard Time
# 440	(UTC+04:00) Abu Dhabi, Muscat
# 441	Arabian Daylight Time
# 442	Arabian Standard Time
# 447	(UTC+04:00) Baku
# 448	Azerbaijan Daylight Time
# 449	Azerbaijan Standard Time
# 450	(UTC+04:00) Yerevan
# 451	Caucasus Daylight Time
# 452	Caucasus Standard Time
# 460	(UTC+04:30) Kabul
# 461	Afghanistan Daylight Time
# 462	Afghanistan Standard Time
# 470	(UTC+05:00) Ekaterinburg
# 471	Ekaterinburg Daylight Time
# 472	Ekaterinburg Standard Time
# 480	(UTC+05:00) Islamabad, Karachi, Tashkent
# 481	West Asia Daylight Time
# 482	West Asia Standard Time
# 490	(UTC+05:30) Chennai, Kolkata, Mumbai, New Delhi
# 491	India Daylight Time
# 492	India Standard Time
# 500	(UTC+05:45) Kathmandu
# 501	Nepal Daylight Time
# 502	Nepal Standard Time
# 510	(UTC+06:00) Astana, Dhaka
# 511	Central Asia Daylight Time
# 512	Central Asia Standard Time
# 520	(UTC+06:00) Almaty, Novosibirsk
# 521	N. Central Asia Daylight Time
# 522	N. Central Asia Standard Time
# 530	(UTC+05:30) Sri Jayawardenepura
# 531	Sri Lanka Daylight Time
# 532	Sri Lanka Standard Time
# 540	(UTC+06:30) Yangon (Rangoon)
# 541	Myanmar Daylight Time
# 542	Myanmar Standard Time
# 550	(UTC+07:00) Krasnoyarsk
# 551	North Asia Daylight Time
# 552	North Asia Standard Time
# 560	(UTC+07:00) Bangkok, Hanoi, Jakarta
# 561	SE Asia Daylight Time
# 562	SE Asia Standard Time
# 570	(UTC+08:00) Beijing, Chongqing, Hong Kong, Urumqi
# 571	China Daylight Time
# 572	China Standard Time
# 580	(UTC+08:00) Irkutsk, Ulaan Bataar
# 581	North Asia East Daylight Time
# 582	North Asia East Standard Time
# 590	(UTC+08:00) Kuala Lumpur, Singapore
# 591	Malay Peninsula Daylight Time
# 592	Malay Peninsula Standard Time
# 600	(UTC+08:00) Taipei
# 601	Taipei Daylight Time
# 602	Taipei Standard Time
# 610	(UTC+08:00) Perth
# 611	W. Australia Daylight Time
# 612	W. Australia Standard Time
# 620	(UTC+09:00) Seoul
# 621	Korea Daylight Time
# 622	Korea Standard Time
# 630	(UTC+09:00) Osaka, Sapporo, Tokyo
# 631	Tokyo Daylight Time
# 632	Tokyo Standard Time
# 640	(UTC+09:00) Yakutsk
# 641	Yakutsk Daylight Time
# 642	Yakutsk Standard Time
# 650	(UTC+09:30) Darwin
# 651	AUS Central Daylight Time
# 652	AUS Central Standard Time
# 660	(UTC+09:30) Adelaide
# 661	Cen. Australia Daylight Time
# 662	Cen. Australia Standard Time
# 670	(UTC+10:00) Canberra, Melbourne, Sydney
# 671	AUS Eastern Daylight Time
# 672	AUS Eastern Standard Time
# 680	(UTC+10:00) Brisbane
# 681	E. Australia Daylight Time
# 682	E. Australia Standard Time
# 690	(UTC+10:00) Hobart
# 691	Tasmania Daylight Time
# 692	Tasmania Standard Time
# 700	(UTC+10:00) Vladivostok
# 701	Vladivostok Daylight Time
# 702	Vladivostok Standard Time
# 710	(UTC+10:00) Guam, Port Moresby
# 711	West Pacific Daylight Time
# 712	West Pacific Standard Time
# 720	(UTC+11:00) Magadan, Solomon Is., New Caledonia
# 721	Central Pacific Daylight Time
# 722	Central Pacific Standard Time
# 730	(UTC+12:00) Fiji, Kamchatka, Marshall Is.
# 731	Fiji Daylight Time
# 732	Fiji Standard Time
# 740	(UTC+12:00) Auckland, Wellington
# 741	New Zealand Daylight Time
# 742	New Zealand Standard Time
# 750	(UTC+13:00) Nuku'alofa
# 751	Tonga Daylight Time
# 752	Tonga Standard Time
# 770	(UTC-03:00) Montevideo
# 771	Montevideo Daylight Time
# 772	Montevideo Standard Time
# 790	(UTC-04:00) La Paz
# 791	SA Western Daylight Time
# 792	SA Western Standard Time
# 810	(UTC-04:30) Caracas
# 811	Venezuela Daylight Time
# 812	Venezuela Standard Time
# 830	(UTC-03:00) Georgetown
# 831	SA Eastern Daylight Time
# 832	SA Eastern Standard Time
# 840	(UTC-03:00) Buenos Aires
# 841	Argentina Daylight Time
# 842	Argentina Standard Time
# 860	(UTC+05:00) Tashkent
# 870	(UTC+05:00) Islamabad, Karachi
# 871	Pakistan Daylight Time
# 872	Pakistan Standard Time
# 880	(UTC) Monrovia, Reykjavik
# 890	(UTC) Casablanca
# 891	Morocco Daylight Time
# 892	Morocco Standard Time
# 910	(UTC+04:00) Port Louis
# 911	Mauritius Daylight Time
# 912	Mauritius Standard Time
# 930	(UTC) Coordinated Universal Time
# 931	Coordinated Universal Time
# 932	Coordinated Universal Time
# 940	(UTC-03:00) Cayenne
# 950	(UTC-04:00) Georgetown, La Paz, San Juan
# 960	(UTC-04:00) Asuncion
# 961	Paraguay Daylight Time
# 962	Paraguay Standard Time
# 970	(UTC+12:00) Fiji, Marshall Is.
# 980	(UTC+12:00) Petropavlovsk-Kamchatsky
# 981	Kamchatka Daylight Time
# 982	Kamchatka Standard Time
# 990	(UTC-05:00) Bogota, Lima, Quito
# 1010	(UTC+06:00) Astana
# 1020	(UTC+06:00) Dhaka
# 1021	Bangladesh Daylight Time
# 1022	Bangladesh Standard Time
# 1030	(UTC+08:00) Irkutsk
# 1040	(UTC+08:00) Ulaanbaatar
# 1041	Ulaanbaatar Daylight Time
# 1042	Ulaanbaatar Standard Time
# 1050	(UTC-11:00) Samoa
# 1060	(UTC-11:00) Midway Islands
# 1061	North Pacific Daylight Time
# 1062	North Pacific Standard Time
# 1070	(UTC+04:00) Tbilisi
# 1080	(UTC+06:00) Novosibirsk
# 1100	(UTC-08:00) Baja California
# 1110	(UTC-03:00) Cayenne, Fortaleza
# 1120	(UTC-04:00) Cuiaba
# 1130	(UTC-04:00) Georgetown, La Paz, Manaus, San Juan
# 1140	(UTC+12:00) Fiji
# 1150	(UTC-01:00) Coordinated Universal Time-01
# 1151	UTC-01
# 1152	UTC-01
# 1160	(UTC-02:00) Coordinated Universal Time-02
# 1161	UTC-02
# 1162	UTC-02
# 1170	(UTC-03:00) Coordinated Universal Time-03
# 1171	UTC-03
# 1172	UTC-03
# 1180	(UTC-04:00) Coordinated Universal Time-04
# 1181	UTC-04
# 1182	UTC-04
# 1190	(UTC-05:00) Coordinated Universal Time-05
# 1191	UTC-05
# 1192	UTC-05
# 1200	(UTC-06:00) Coordinated Universal Time-06
# 1201	UTC-06
# 1202	UTC-06
# 1210	(UTC-07:00) Coordinated Universal Time-07
# 1211	UTC-07
# 1212	UTC-07
# 1220	(UTC-08:00) Coordinated Universal Time-08
# 1221	UTC-08
# 1222	UTC-08
# 1230	(UTC-09:00) Coordinated Universal Time-09
# 1231	UTC-09
# 1232	UTC-09
# 1240	(UTC-10:00) Coordinated Universal Time-10
# 1241	UTC-10
# 1242	UTC-10
# 1250	(UTC-11:00) Coordinated Universal Time-11
# 1251	UTC-11
# 1252	UTC-11
# 1260	(UTC-12:00) Coordinated Universal Time-12
# 1261	UTC-12
# 1262	UTC-12
# 1270	(UTC+01:00) Coordinated Universal Time+01
# 1271	UTC+01
# 1272	UTC+01
# 1280	(UTC+02:00) Coordinated Universal Time+02
# 1281	UTC+02
# 1282	UTC+02
# 1290	(UTC+03:00) Coordinated Universal Time+03
# 1291	UTC+03
# 1292	UTC+03
# 1300	(UTC+04:00) Coordinated Universal Time+04
# 1301	UTC+04
# 1302	UTC+04
# 1310	(UTC+05:00) Coordinated Universal Time+05
# 1311	UTC+05
# 1312	UTC+05
# 1320	(UTC+06:00) Coordinated Universal Time+06
# 1321	UTC+06
# 1322	UTC+06
# 1330	(UTC+07:00) Coordinated Universal Time+07
# 1331	UTC+07
# 1332	UTC+07
# 1340	(UTC+08:00) Coordinated Universal Time+08
# 1341	UTC+08
# 1342	UTC+08
# 1350	(UTC+09:00) Coordinated Universal Time+09
# 1351	UTC+09
# 1352	UTC+09
# 1360	(UTC+10:00) Coordinated Universal Time+10
# 1361	UTC+10
# 1362	UTC+10
# 1370	(UTC+11:00) Coordinated Universal Time+11
# 1371	UTC+11
# 1372	UTC+11
# 1380	(UTC+12:00) Coordinated Universal Time+12
# 1381	UTC+12
# 1382	UTC+12
# 1390	(UTC+13:00) Coordinated Universal Time+13
# 1391	UTC+13
# 1392	UTC+13
# 1410	(UTC+02:00) Damascus
# 1411	Syria Daylight Time
# 1412	Syria Standard Time
# 1420	(UTC+12:00) Petropavlovsk-Kamchatsky - Old
# 1440	(UTC+01:00) Windhoek
# 1460	(UTC+11:00) Solomon Is., New Caledonia
# 1470	(UTC+11:00) Magadan
# 1471	Magadan Daylight Time
# 1472	Magadan Standard Time
# 1490	(UTC+02:00) Athens, Bucharest
# 1500	(UTC+02:00) Istanbul
# 1501	Turkey Daylight Time
# 1502	Turkey Standard Time
# 1520	(UTC+04:00) Moscow, St. Petersburg, Volgograd
# 1530	(UTC+06:00) Ekaterinburg
# 1540	(UTC+07:00) Novosibirsk
# 1550	(UTC+08:00) Krasnoyarsk
# 1560	(UTC+09:00) Irkutsk
# 1570	(UTC+10:00) Yakutsk
# 1580	(UTC+11:00) Vladivostok
# 1590	(UTC+12:00) Magadan
# 1600	(UTC+03:00) Kaliningrad
# 1601	Kaliningrad Daylight Time
# 1602	Kaliningrad Standard Time
# 1620	(UTC+03:00) Kaliningrad, Minsk
# 1630	(UTC+02:00) Nicosia
# 1640	(UTC+13:00) Samoa
# 1660	(UTC-03:00) Salvador
# 1661	Bahia Daylight Time
# 1662	Bahia Standard Time
# 1680	(UTC+02:00) E. Europe
# 1700	(UTC+03:00) Amman
# 1720	(UTC+02:00) Tripoli
# 1721	Libya Daylight Time
# 1722	Libya Standard Time
# 1740	(UTC+05:00) Ashgabat, Tashkent
# 1760	(UTC-02:00) Mid-Atlantic - Old



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Mac wifi.log parser."""

import unittest

from plaso.formatters import mac_wifi  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import mac_wifi

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class MacWifiUnitTest(test_lib.ParserTestCase):
  """Tests for the Mac wifi.log parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'wifi_turned_over.log'])
  def testParseTurnedOver(self):
    """Tests the Parse function."""
    parser = mac_wifi.MacWifiLogParser()
    knowledge_base_values = {u'year': 2017}
    storage_writer = self._ParseFile(
        [u'wifi_turned_over.log'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 6)

    events = list(storage_writer.GetEvents())

    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2017-01-02 00:10:15')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(
        event.text,
        u'test-macbookpro newsyslog[50498]: logfile turned over')

    event = events[1]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2017-01-02 00:11:02.378')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_text = (
        u'<kernel> wl0: powerChange: *** '
        u'BONJOUR/MDNS OFFLOADS ARE NOT RUNNING.')
    self.assertEqual(event.text, expected_text)

    event = events[2]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2017-01-02 07:41:01.371')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_string = (
        u'<kernel> wl0: leaveModulePoweredForOffloads: Wi-Fi will stay on.')
    self.assertEqual(event.text, expected_string)

    event = events[5]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2017-01-02 07:41:02.207')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_text = (
        u'<kernel> Setting BTCoex Config: enable_2G:1, profile_2g:0, '
        u'enable_5G:1, profile_5G:0')

    self.assertEqual(event.text, expected_text)

  @shared_test_lib.skipUnlessHasTestFile([u'wifi.log'])
  def testParse(self):
    """Tests the Parse function."""
    parser = mac_wifi.MacWifiLogParser()
    knowledge_base_values = {u'year': 2013}
    storage_writer = self._ParseFile(
        [u'wifi.log'], parser,
        knowledge_base_values=knowledge_base_values)

    self.assertEqual(storage_writer.number_of_events, 10)

    events = list(storage_writer.GetEvents())

    event = events[1]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-14 20:36:37.222')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.agent, u'airportd[88]')
    self.assertEqual(event.function, u'airportdProcessDLILEvent')
    self.assertEqual(event.action, u'Interface en0 turn up.')
    self.assertEqual(event.text, u'en0 attached (up)')

    expected_message = (
        u'Action: Interface en0 turn up. '
        u'Agent: airportd[88] '
        u'(airportdProcessDLILEvent) '
        u'Log: en0 attached (up)')
    expected_short_message = (
        u'Action: Interface en0 turn up.')

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[2]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-14 20:36:43.818')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.agent, u'airportd[88]')
    self.assertEqual(event.function, u'_doAutoJoin')
    self.assertEqual(event.action, u'Wifi connected to SSID CampusNet')

    expected_text = (
        u'Already associated to \u201cCampusNet\u201d. Bailing on auto-join.')
    self.assertEqual(event.text, expected_text)

    event = events[3]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-14 21:50:52.395')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_string = (
        u'<airportd[88]> _handleLinkEvent: Unable to process link event, '
        u'op mode request returned -3903 (Operation not supported)')

    self.assertEqual(event.text, expected_string)

    event = events[6]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-11-14 21:52:09.883')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(u'airportd[88]', event.agent)
    self.assertEqual(u'_processSystemPSKAssoc', event.function)

    expected_action = (
        u'New wifi configured. BSSID: 88:30:8a:7a:61:88, SSID: AndroidAP, '
        u'Security: WPA2 Personal.')

    self.assertEqual(event.action, expected_action)

    expected_text = (
        u'No password for network <CWNetwork: 0x7fdfe970b250> '
        u'[ssid=AndroidAP, bssid=88:30:8a:7a:61:88, security=WPA2 '
        u'Personal, rssi=-21, channel=<CWChannel: 0x7fdfe9712870> '
        u'[channelNumber=11(2GHz), channelWidth={20MHz}], ibss=0] '
        u'in the system keychain')

    self.assertEqual(event.text, expected_text)

    event = events[8]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-12-31 23:59:38.165')
    self.assertEqual(event.timestamp, expected_timestamp)

    event = events[9]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-01-01 01:12:17.311')
    self.assertEqual(event.timestamp, expected_timestamp)

if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Network Drives Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import network_drives

from tests.parsers.winreg_plugins import test_lib


class NetworkDrivesPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Network Drives Windows Registry plugin."""

  def _CreateTestKey(self, key_path, time_string):
    """Creates Registry keys and values for testing.

    Args:
      key_path (str): Windows Registry key path.
      time_string (str): key last written date and time.

    Returns:
      dfwinreg.WinRegistryKey: Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromString(time_string)
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        u'Network', key_path=key_path,
        last_written_time=filetime.timestamp, offset=153)

    # Setup H drive.
    h_key = dfwinreg_fake.FakeWinRegistryKey(
        u'H', last_written_time=filetime.timestamp)
    registry_key.AddSubkey(h_key)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ConnectionType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x04'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'DeferFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ProviderFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    value_data = u'Microsoft Windows Network'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ProviderName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    h_key.AddValue(registry_value)

    value_data = b'\x00\x02\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ProviderType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    value_data = u'\\\\acme.local\\Shares\\User_Data\\John.Doe'.encode(
        u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'RemotePath', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    h_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'UserName', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    h_key.AddValue(registry_value)

    # Setup Z drive.
    z_key = dfwinreg_fake.FakeWinRegistryKey(
        u'Z', last_written_time=filetime.timestamp)
    registry_key.AddSubkey(z_key)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ConnectionType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x04'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'DeferFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ProviderFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    value_data = u'Microsoft Windows Network'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ProviderName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    z_key.AddValue(registry_value)

    value_data = b'\x00\x02\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ProviderType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    value_data = u'\\\\secret_computer\\Media'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'RemotePath', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    z_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'UserName', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    z_key.AddValue(registry_value)

    return registry_key

  def testProcess(self):
    """Tests the Process function on created key."""
    key_path = u'HKEY_CURRENT_USER\\Network'
    time_string = u'2013-01-30 10:47:57'
    registry_key = self._CreateTestKey(key_path, time_string)

    plugin = network_drives.NetworkDrivesPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    self.assertEqual(storage_writer.number_of_events, 2)

    events = list(storage_writer.GetSortedEvents())

    event_object = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(time_string)
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_message = (
        u'[{0:s}] '
        u'DriveLetter: H '
        u'RemoteServer: acme.local '
        u'ShareName: \\Shares\\User_Data\\John.Doe '
        u'Type: Mapped Drive').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(
        event_object, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Windows services event formatter.

The Windows services are derived from Windows Registry files.
"""

from plaso.formatters import manager
from plaso.formatters import winreg
from plaso.winnt import human_readable_service_enums


class WinRegistryServiceFormatter(winreg.WinRegistryGenericFormatter):
  """Formatter for a Windows service event."""

  DATA_TYPE = u'windows:registry:service'

  def GetMessages(self, formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    regvalue = getattr(event, u'regvalue', {})
    # Loop over all the registry value names in the service key.
    for service_value_name in regvalue.keys():
      # A temporary variable so we can refer to this long name more easily.
      service_enums = human_readable_service_enums.SERVICE_ENUMS
      # Check if we need to can make the value more human readable.
      if service_value_name in service_enums.keys():
        service_enum = service_enums[service_value_name]
        # Find the human readable version of the name and fall back to the
        # raw value if it's not found.
        human_readable_value = service_enum.get(
            regvalue[service_value_name],
            regvalue[service_value_name])
        regvalue[service_value_name] = human_readable_value

    return super(WinRegistryServiceFormatter, self).GetMessages(
        formatter_mediator, event)


manager.FormattersManager.RegisterFormatter(WinRegistryServiceFormatter)



# -*- coding: utf-8 -*-
"""Parser for the Mac OS X Document Versions files."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class MacDocumentVersionsEventData(events.EventData):
  """Mac OS X Document Versions database event data.

  Attributes:
    name (str): name of the original file.
    path (str): path from the original file.
    version_path (str): path to the version copy of the original file.
    last_time (str): the system user ID of the user that opened the file.
    user_sid (str): identification user ID that open the file.
  """

  DATA_TYPE = u'mac:document_versions:file'

  def __init__(self):
    """Initializes event data."""
    super(MacDocumentVersionsEventData, self).__init__(data_type=self.DATA_TYPE)
    # TODO: shouldn't this be a separate event?
    self.last_time = None
    self.name = None
    self.path = None
    self.user_sid = None
    self.version_path = None


class MacDocumentVersionsPlugin(interface.SQLitePlugin):
  """Parse the Mac OS X Document Versions SQLite database.."""

  NAME = u'mac_document_versions'
  DESCRIPTION = u'Parser for document revisions SQLite database files.'

  # Define the needed queries.
  # name: name from the original file.
  # path: path from the original file (include the file)
  # last_time: last time when the file was replicated.
  # version_path: path where the version is stored.
  # version_time: the timestamp when the version was created.
  QUERIES = [
      ((u'SELECT f.file_name AS name, f.file_path AS path, '
        u'f.file_last_seen AS last_time, g.generation_path AS version_path, '
        u'g.generation_add_time AS version_time FROM files f, generations g '
        u'WHERE f.file_storage_id = g.generation_storage_id;'),
       u'DocumentVersionsRow')]

  # The required tables for the query.
  REQUIRED_TABLES = frozenset([u'files', u'generations'])

  SCHEMAS = [{
      u'files': (
          u'CREATE TABLE files (file_row_id INTEGER PRIMARY KEY ASC, file_name '
          u'TEXT, file_parent_id INTEGER, file_path TEXT, file_inode INTEGER, '
          u'file_last_seen INTEGER NOT NULL DEFAULT 0, file_status INTEGER NOT '
          u'NULL DEFAULT 1, file_storage_id INTEGER NOT NULL)'),
      u'generations': (
          u'CREATE TABLE generations (generation_id INTEGER PRIMARY KEY ASC, '
          u'generation_storage_id INTEGER NOT NULL, generation_name TEXT NOT '
          u'NULL, generation_client_id TEXT NOT NULL, generation_path TEXT '
          u'UNIQUE, generation_options INTEGER NOT NULL DEFAULT 1, '
          u'generation_status INTEGER NOT NULL DEFAULT 1, generation_add_time '
          u'INTEGER NOT NULL DEFAULT 0, generation_size INTEGER NOT NULL '
          u'DEFAULT 0, generation_prunable INTEGER NOT NULL DEFAULT 0)'),
      u'storage': (
          u'CREATE TABLE storage (storage_id INTEGER PRIMARY KEY ASC '
          u'AUTOINCREMENT, storage_options INTEGER NOT NULL DEFAULT 1, '
          u'storage_status INTEGER NOT NULL DEFAULT 1)')}]

  # The SQL field path is the relative path from DocumentRevisions.
  # For this reason the Path to the program has to be added at the beginning.
  ROOT_VERSION_PATH = u'/.DocumentRevisions-V100/'

  def DocumentVersionsRow(
      self, parser_mediator, row, query=None, **unused_kwargs):
    """Parses a document versions row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      row (sqlite3.Row): row.
      query (Optional[str]): query.
    """
    # Note that pysqlite does not accept a Unicode string in row['string'] and
    # will raise "IndexError: Index must be int or string".

    # version_path = "PerUser/UserID/xx/client_id/version_file"
    # where PerUser and UserID are a real directories.
    paths = row['version_path'].split(u'/')
    if len(paths) < 2 or not paths[1].isdigit():
      user_sid = u''
    else:
      user_sid = paths[1]
    version_path = self.ROOT_VERSION_PATH + row['version_path']
    path, _, _ = row['path'].rpartition(u'/')

    event_data = MacDocumentVersionsEventData()
    # TODO: shouldn't this be a separate event?
    event_data.last_time = row['last_time']
    event_data.name = row['name']
    event_data.path = path
    event_data.query = query
    # Note that the user_sid value is expected to be a string.
    event_data.user_sid = u'{0!s}'.format(user_sid)
    event_data.version_path = version_path

    date_time = dfdatetime_posix_time.PosixTime(timestamp=row['version_time'])
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)
    parser_mediator.ProduceEventWithEventData(event, event_data)


sqlite.SQLiteParser.RegisterPlugin(MacDocumentVersionsPlugin)



# -*- coding: utf-8 -*-
"""CLI argument helpers."""

from plaso.cli.helpers import analysis_plugins
from plaso.cli.helpers import artifact_definitions
from plaso.cli.helpers import data_location
from plaso.cli.helpers import date_filters
from plaso.cli.helpers import dynamic_output
from plaso.cli.helpers import elastic_output
from plaso.cli.helpers import event_filters
from plaso.cli.helpers import extraction
from plaso.cli.helpers import filter_file
from plaso.cli.helpers import hashers
from plaso.cli.helpers import language
from plaso.cli.helpers import mysql_4n6time_output
from plaso.cli.helpers import nsrlsvr_analysis
from plaso.cli.helpers import output_modules
from plaso.cli.helpers import parsers
from plaso.cli.helpers import sqlite_4n6time_output
from plaso.cli.helpers import status_view
from plaso.cli.helpers import storage_file
from plaso.cli.helpers import storage_format
from plaso.cli.helpers import tagging_analysis
from plaso.cli.helpers import temporary_directory
from plaso.cli.helpers import text_prepend
from plaso.cli.helpers import timesketch_output
from plaso.cli.helpers import viper_analysis
from plaso.cli.helpers import virustotal_analysis
from plaso.cli.helpers import windows_services_analysis
from plaso.cli.helpers import xlsx_output
from plaso.cli.helpers import yara_rules



# -*- coding: utf-8 -*-
"""The SCCM log formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class SCCMEventFormatter(interface.ConditionalEventFormatter):
  """Class for SCCM event formatter."""

  DATA_TYPE = u'software_management:sccm:log'

  FORMAT_STRING_SEPARATOR = u' '

  FORMAT_STRING_PIECES = [
      u'{component}',
      u'{text}']

  FORMAT_STRING_SHORT_PIECES = [u'{text}']

  SOURCE_LONG = u'SCCM Event'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(SCCMEventFormatter)



# -*- coding: utf-8 -*-
"""The syslog cron formatters."""

from plaso.formatters import interface
from plaso.formatters import manager


class CronTaskRunEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a syslog cron task run event."""

  DATA_TYPE = u'syslog:cron:task_run'

  FORMAT_STRING_SEPARATOR = u' '

  FORMAT_STRING_PIECES = [
      u'Cron ran: {command}',
      u'for user: {username}',
      u'pid: {pid}']

  FORMAT_STRING_SHORT = u'{body}'

  SOURCE_LONG = u'Cron log'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(CronTaskRunEventFormatter)



# -*- coding: utf-8 -*-
"""A plugin that extracts browser history from events."""

import collections
import logging
import re
import urllib

from plaso.analysis import interface
from plaso.analysis import manager
from plaso.containers import reports
from plaso.formatters import manager as formatters_manager
from plaso.lib import py2to3


# Create a lightweight object that is used to store timeline based information
# about each search term.
SEARCH_OBJECT = collections.namedtuple(
    u'SEARCH_OBJECT', u'time source engine search_term')


class BrowserSearchPlugin(interface.AnalysisPlugin):
  """Analyze browser search entries from events."""

  NAME = u'browser_search'

  # Indicate that we do not want to run this plugin during regular extraction.
  ENABLE_IN_EXTRACTION = False

  _EVENT_TAG_COMMENT = u'Browser Search'
  _EVENT_TAG_LABELS = [u'browser_search']

  # TODO: use groups to build a single RE.

  # Here we define filters and callback methods for all hits on each filter.
  _URL_FILTERS = frozenset([
      (u'Bing', re.compile(u'bing.com/search'), u'_ExtractSearchQueryFromURL'),
      (u'DuckDuckGo', re.compile(r'duckduckgo\.com'),
       u'_ExtractDuckDuckGoSearchQuery'),
      (u'GMail', re.compile(r'mail\.google\.com'),
       u'_ExtractGMailSearchQuery'),
      (u'Google Docs', re.compile(r'docs.google.com'),
       u'_ExtractGoogleDocsSearchQuery'),
      (u'Google Drive', re.compile(r'drive\.google\.com/drive/search'),
       u'_ExtractGoogleSearchQuery'),
      (u'Google Search',
       re.compile(r'(www\.|encrypted\.|/)google\.[^/]*/search'),
       u'_ExtractGoogleSearchQuery'),
      (u'Google Sites', re.compile(r'sites.google.com/site'),
       u'_ExtractGoogleSearchQuery'),
      (u'Yandex', re.compile(r'yandex\.com/search'),
       u'_ExtractYandexSearchQuery'),
      (u'Youtube', re.compile(r'youtube\.com'),
       '_ExtractYouTubeSearchQuery'),
  ])

  def __init__(self):
    """Initializes an analysis plugin."""
    super(BrowserSearchPlugin, self).__init__()
    self._counter = collections.Counter()

    # Store a list of search terms in a timeline format.
    # The format is key = timestamp, value = (source, engine, search term).
    self._search_term_timeline = []

  def _DecodeURL(self, url):
    """Decodes the URL, replaces %XX to their corresponding characters.

    Args:
      url (str): encoded URL.

    Returns:
      str: decoded URL.
    """
    if not url:
      return u''

    decoded_url = urllib.unquote(url)
    if isinstance(decoded_url, py2to3.BYTES_TYPE):
      try:
        decoded_url = decoded_url.decode(u'utf-8')
      except UnicodeDecodeError as exception:
        decoded_url = decoded_url.decode(u'utf-8', errors=u'replace')
        logging.warning(
            u'Unable to decode URL: {0:s} with error: {1:s}'.format(
                url, exception))

    return decoded_url

  def _ExtractDuckDuckGoSearchQuery(self, url):
    """Extracts a search query from a DuckDuckGo search URL.

    DuckDuckGo: https://duckduckgo.com/?q=query

    Args:
      url (str): URL.

    Returns:
      str: search query.
    """
    if u'q=' not in url:
      return

    return self._GetBetweenQEqualsAndAmbersand(url).replace(u'+', u' ')

  def _ExtractGMailSearchQuery(self, url):
    """Extracts a search query from a GMail search URL.

    GMail: https://mail.google.com/mail/u/0/#search/query[/?]

    Args:
      url (str): URL.

    Returns:
      str: search query.
    """
    if u'search/' not in url:
      return

    _, _, line = url.partition(u'search/')
    line, _, _ = line.partition(u'/')
    line, _, _ = line.partition(u'?')

    return line.replace(u'+', u' ')

  def _ExtractGoogleDocsSearchQuery(self, url):
    """Extracts a search query from a Google docs URL.

    Google Docs: https://docs.google.com/.*/u/0/?q=query

    Args:
      url (str): URL.

    Returns:
      str: search query.
    """
    if u'q=' not in url:
      return

    line = self._GetBetweenQEqualsAndAmbersand(url)
    if not line:
      return

    return line.replace(u'+', u' ')

  def _ExtractGoogleSearchQuery(self, url):
    """Extracts a search query from a Google URL.

    Google Drive: https://drive.google.com/drive/search?q=query
    Google Search: https://www.google.com/search?q=query
    Google Sites: https://sites.google.com/site/.*/system/app/pages/
                  search?q=query

    Args:
      url (str): URL.

    Returns:
      str: search query.
    """
    if u'search' not in url or u'q=' not in url:
      return

    line = self._GetBetweenQEqualsAndAmbersand(url)
    if not line:
      return

    return line.replace(u'+', u' ')

  def _ExtractYandexSearchQuery(self, url):
    """Extracts a search query from a Yandex search URL.

    Yandex: https://www.yandex.com/search/?text=query

    Args:
      url (str): URL.

    Returns:
      str: search query.
    """
    if u'text=' not in url:
      return
    _, _, line = url.partition(u'text=')
    before_and, _, _ = line.partition(u'&')
    if not before_and:
      return
    yandex_search_url = before_and.split()[0]

    return yandex_search_url.replace(u'+', u' ')

  def _ExtractYouTubeSearchQuery(self, url):
    """Extracts a search query from a YouTube search URL.

    YouTube: https://www.youtube.com/results?search_query=query

    Args:
      url (str): URL.

    Returns:
      str: search query.
    """
    return self._ExtractSearchQueryFromURL(url)

  def _ExtractSearchQueryFromURL(self, url):
    """Extracts a search query from the URL.

    Bing: https://www.bing.com/search?q=query
    GitHub: https://github.com/search?q=query

    Args:
      url (str): URL.

    Returns:
      str: search query, the value between 'q=' and '&'.
    """
    if u'search' not in url or u'q=' not in url:
      return

    return self._GetBetweenQEqualsAndAmbersand(url).replace(u'+', u' ')

  def _GetBetweenQEqualsAndAmbersand(self, url):
    """Retrieves the substring between the substrings 'q=' and '&'.

    Args:
      url (str): URL.

    Returns:
      str: search query, the value between 'q=' and '&'.
    """
    # Make sure we're analyzing the query part of the URL.
    _, _, url = url.partition(u'?')
    # Look for a key value pair named 'q'.
    _, _, url = url.partition(u'q=')
    if not url:
      return u''

    # Strip additional key value pairs.
    url, _, _ = url.partition(u'&')
    return url

  def CompileReport(self, mediator):
    """Compiles an analysis report.

    Args:
      mediator (AnalysisMediator): mediates interactions between
          analysis plugins and other components, such as storage and dfvfs.

    Returns:
      AnalysisReport: analysis report.
    """
    results = {}
    for key, count in iter(self._counter.items()):
      search_engine, _, search_term = key.partition(u':')
      results.setdefault(search_engine, {})
      results[search_engine][search_term] = count

    lines_of_text = []
    for search_engine, terms in sorted(results.items()):
      lines_of_text.append(u' == ENGINE: {0:s} =='.format(search_engine))

      for search_term, count in sorted(
          terms.items(), key=lambda x: (x[1], x[0]), reverse=True):
        lines_of_text.append(u'{0:d} {1:s}'.format(count, search_term))

      # An empty string is added to have SetText create an empty line.
      lines_of_text.append(u'')

    lines_of_text.append(u'')
    report_text = u'\n'.join(lines_of_text)
    analysis_report = reports.AnalysisReport(
        plugin_name=self.NAME, text=report_text)
    analysis_report.report_array = self._search_term_timeline
    analysis_report.report_dict = results
    return analysis_report

  def ExamineEvent(self, mediator, event):
    """Analyzes an event.

    Args:
      mediator (AnalysisMediator): mediates interactions between
          analysis plugins and other components, such as storage and dfvfs.
      event (EventObject): event to examine.
    """
    # This event requires an URL attribute.
    url = getattr(event, u'url', None)
    if not url:
      return

    # TODO: refactor this the source should be used in formatting only.
    # Check if we are dealing with a web history event.
    source, _ = formatters_manager.FormattersManager.GetSourceStrings(event)

    if source != u'WEBHIST':
      return

    for engine, url_expression, method_name in self._URL_FILTERS:
      callback_method = getattr(self, method_name, None)
      if not callback_method:
        logging.warning(u'Missing method: {0:s}'.format(callback_method))
        continue

      match = url_expression.search(url)
      if not match:
        continue

      search_query = callback_method(url)
      if not search_query:
        logging.warning(u'Missing search query for URL: {0:s}'.format(url))
        continue

      search_query = self._DecodeURL(search_query)
      if not search_query:
        continue

      event_tag = self._CreateEventTag(
          event, self._EVENT_TAG_COMMENT, self._EVENT_TAG_LABELS)
      mediator.ProduceEventTag(event_tag)

      self._counter[u'{0:s}:{1:s}'.format(engine, search_query)] += 1

      # Add the timeline format for each search term.
      timestamp = getattr(event, u'timestamp', 0)
      source = getattr(event, u'parser', u'N/A')
      source = getattr(event, u'plugin', source)
      self._search_term_timeline.append(
          SEARCH_OBJECT(timestamp, source, engine, search_query))


manager.AnalysisPluginManager.RegisterPlugin(BrowserSearchPlugin)



# -*- coding: utf-8 -*-
"""The McAfee AV Logs file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class McafeeAccessProtectionLogEventFormatter(
    interface.ConditionalEventFormatter):
  """Formatter for a McAfee Access Protection Log event."""

  DATA_TYPE = u'av:mcafee:accessprotectionlog'

  FORMAT_STRING_PIECES = [
      u'File Name: {filename}',
      u'User: {username}',
      u'{trigger_location}',
      u'{status}',
      u'{rule}',
      u'{action}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{filename}',
      u'{action}']

  SOURCE_LONG = u'McAfee Access Protection Log'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(
    McafeeAccessProtectionLogEventFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Keychain password database parser."""

import unittest

from plaso.formatters import mac_keychain  # pylint: disable=unused-import
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import mac_keychain

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class MacKeychainParserTest(test_lib.ParserTestCase):
  """Tests for keychain file parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'login.keychain'])
  def testParse(self):
    """Tests the Parse function."""
    parser = mac_keychain.KeychainParser()
    storage_writer = self._ParseFile([u'login.keychain'], parser)

    self.assertEqual(storage_writer.number_of_events, 8)

    events = list(storage_writer.GetEvents())

    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-01-26 14:51:48')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(
        event.timestamp_desc, definitions.TIME_DESCRIPTION_CREATION)

    self.assertEqual(event.entry_name, u'Secret Application')
    self.assertEqual(event.account_name, u'moxilo')
    expected_ssgp = (
        u'b8e44863af1cb0785b89681d22e2721997ccfb8adb8853e726aff94c8830b05a')
    self.assertEqual(event.ssgp_hash, expected_ssgp)
    self.assertEqual(event.text_description, u'N/A')
    expected_message = u'Name: Secret Application Account: moxilo'
    expected_short_message = u'Secret Application'
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[1]

    self.assertEqual(
        event.timestamp_desc,
        definitions.TIME_DESCRIPTION_MODIFICATION)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-01-26 14:52:29')
    self.assertEqual(event.timestamp, expected_timestamp)

    event = events[2]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-01-26 14:53:29')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.entry_name, u'Secret Note')
    self.assertEqual(event.text_description, u'secure note')
    self.assertEqual(len(event.ssgp_hash), 1696)
    expected_message = u'Name: Secret Note'
    expected_short_message = u'Secret Note'
    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[4]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-01-26 14:54:33')
    self.assertEqual(event.timestamp, expected_timestamp)

    self.assertEqual(event.entry_name, u'plaso.kiddaland.net')
    self.assertEqual(event.account_name, u'MrMoreno')
    expected_ssgp = (
        u'83ccacf55a8cb656d340ec405e9d8b308fac54bb79c5c9b0219bd0d700c3c521')
    self.assertEqual(event.ssgp_hash, expected_ssgp)
    self.assertEqual(event.where, u'plaso.kiddaland.net')
    self.assertEqual(event.protocol, u'http')
    self.assertEqual(event.type_protocol, u'dflt')
    self.assertEqual(event.text_description, u'N/A')

    expected_message = (
        u'Name: plaso.kiddaland.net '
        u'Account: MrMoreno '
        u'Where: plaso.kiddaland.net '
        u'Protocol: http (dflt)')
    expected_short_message = u'plaso.kiddaland.net'
    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests the multi-processing queue."""

import unittest

from plaso.engine import plaso_queue
from plaso.lib import errors
from plaso.multi_processing import multi_process_queue

from tests import test_lib as shared_test_lib


class TestQueueConsumer(object):
  """Class that implements the test queue consumer.

  The queue consumer subscribes to updates on the queue.

  Attributes:
    items (list[object]): queued items.
  """

  def __init__(self, queue):
    """Initializes the queue consumer.

    Args:
      queue (Queue): queue.
    """
    super(TestQueueConsumer, self).__init__()
    self._abort = False
    self._queue = queue
    self.items = []

  @property
  def number_of_items(self):
    """The number of items."""
    return len(self.items)

  def ConsumeItems(self):
    """Consumes the items that are pushed on the queue."""
    while not self._abort:
      try:
        item = self._queue.PopItem()

      except (errors.QueueClose, errors.QueueEmpty):
        break

      if isinstance(item, plaso_queue.QueueAbort):
        break

      self.items.append(item)


class MultiProcessingQueueTest(shared_test_lib.BaseTestCase):
  """Tests the multi-processing queue object."""

  _ITEMS = frozenset([u'item1', u'item2', u'item3', u'item4'])

  def testPushPopItem(self):
    """Tests the PushItem and PopItem functions."""
    # A timeout is used to prevent the multi processing queue to close and
    # stop blocking the current process
    test_queue = multi_process_queue.MultiProcessingQueue(timeout=0.1)

    for item in self._ITEMS:
      test_queue.PushItem(item)

    test_queue_consumer = TestQueueConsumer(test_queue)
    test_queue_consumer.ConsumeItems()

    self.assertEqual(test_queue_consumer.number_of_items, len(self._ITEMS))


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Windows Recycler/Recycle Bin formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class WinRecyclerFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Windows Recycler/Recycle Bin file event."""

  DATA_TYPE = u'windows:metadata:deleted_item'

  _DRIVE_LETTER = {
      0x00: u'A',
      0x01: u'B',
      0x02: u'C',
      0x03: u'D',
      0x04: u'E',
      0x05: u'F',
      0x06: u'G',
      0x07: u'H',
      0x08: u'I',
      0x09: u'J',
      0x0A: u'K',
      0x0B: u'L',
      0x0C: u'M',
      0x0D: u'N',
      0x0E: u'O',
      0x0F: u'P',
      0x10: u'Q',
      0x11: u'R',
      0x12: u'S',
      0x13: u'T',
      0x14: u'U',
      0x15: u'V',
      0x16: u'W',
      0x17: u'X',
      0x18: u'Y',
      0x19: u'Z',
  }

  # The format string.
  FORMAT_STRING_PIECES = [
      u'DC{record_index} ->',
      u'{original_filename}',
      u'[{short_filename}]',
      u'(from drive: {drive_letter})']

  FORMAT_STRING_SHORT_PIECES = [
      u'Deleted file: {original_filename}']

  SOURCE_LONG = u'Recycle Bin'
  SOURCE_SHORT = u'RECBIN'

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    drive_number = event_values.get(u'drive_number', None)
    event_values[u'drive_letter'] = self._DRIVE_LETTER.get(
        drive_number, u'UNKNOWN')

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(WinRecyclerFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the CUPS IPP file event formatter."""

import unittest

from plaso.formatters import cups_ipp

from tests.formatters import test_lib


class CupsIppFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the CUPS IPP event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = cups_ipp.CupsIppFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = cups_ipp.CupsIppFormatter()

    expected_attribute_names = [
        u'status', u'user', u'owner', u'job_name', u'application', u'type_doc',
        u'printer_id']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the xchatscrollback log parser."""

import unittest

from plaso.formatters import xchatscrollback  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import xchatscrollback

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


__author__ = 'Francesco Picasso (francesco.picasso@gmail.com)'


class XChatScrollbackUnitTest(test_lib.ParserTestCase):
  """Tests for the xchatscrollback log parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'xchatscrollback.log'])
  def testParse(self):
    """Tests the Parse function."""
    parser = xchatscrollback.XChatScrollbackParser()
    storage_writer = self._ParseFile([u'xchatscrollback.log'], parser)

    self.assertEqual(storage_writer.number_of_events, 10)

    events = list(storage_writer.GetEvents())

    event = events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-01-16 02:56:19')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = u'[] * Speaking now on ##plaso##'
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[1]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-01-16 02:56:27')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = u'[] * Joachim \xe8 uscito (Client exited)'
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[2]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-01-18 21:58:36')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = u'[] Tcl interface unloaded'
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[3]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-01-18 21:58:36')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = u'[] Python interface unloaded'
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[5]
    self.assertEqual(event.timestamp, 0)

    expected_message = u'[nickname: fpi] 0 is a good timestamp'
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[6]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-01-26 08:50:56')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = u'[] * Topic of #plasify \xe8: .'
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[7]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-01-26 08:51:02')
    self.assertEqual(event.timestamp, expected_timestamp)

    event = events[8]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-01-26 08:52:12')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = u'[nickname: fpi] Hi Kristinn!'
    self._TestGetMessageStrings(event, expected_message, expected_message)

    event = events[9]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-01-26 08:53:13')
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = u'[nickname: Kristinn] GO AND WRITE PARSERS!!! O_o'
    self._TestGetMessageStrings(event, expected_message, expected_message)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the shared library utils."""

import unittest

from plaso.lib import utils


class UtilsTestCase(unittest.TestCase):
  """The unit test for utils method collection."""

  def testIsText(self):
    """Tests the IsText function."""
    bytes_in = b'this is My Weird ASCII and non whatever string.'
    self.assertTrue(utils.IsText(bytes_in))

    bytes_in = u'Plaso Síar Og Raðar Þessu'
    self.assertTrue(utils.IsText(bytes_in))

    bytes_in = b'\x01\\62LSO\xFF'
    self.assertFalse(utils.IsText(bytes_in))

    bytes_in = b'T\x00h\x00i\x00s\x00\x20\x00'
    self.assertTrue(utils.IsText(bytes_in))

    bytes_in = b'Ascii\x00'
    self.assertTrue(utils.IsText(bytes_in))

    bytes_in = b'Ascii Open then...\x00\x99\x23'
    self.assertFalse(utils.IsText(bytes_in))


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the MD5 hasher."""

import unittest

from plaso.analyzers.hashers import md5

from tests import test_lib as shared_test_lib
from tests.analyzers.hashers import test_lib


class MD5Test(test_lib.HasherTestCase):
  """Tests the MD5 hasher."""

  @shared_test_lib.skipUnlessHasTestFile([u'empty_file'])
  def testFileHashMatchesEmptyFile(self):
    """Tests that hasher matches the hash of an empty file."""
    expected_md5 = u'd41d8cd98f00b204e9800998ecf8427e'

    hasher = md5.MD5Hasher()
    self._AssertTestPathStringDigestMatch(hasher, [u'empty_file'], expected_md5)

    hasher = md5.MD5Hasher()
    self._AssertTestPathBinaryDigestMatch(
        hasher, [u'empty_file'], expected_md5.decode(u'hex'))

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testFileHashMatchesKnownFile(self):
    """Tests that hasher matches the hash of a known file."""
    expected_md5 = u'd73c51f10c7ee6a681b7b619ccc6f1c4'

    hasher = md5.MD5Hasher()
    self._AssertTestPathStringDigestMatch(hasher, [u'ímynd.dd'], expected_md5)

    hasher = md5.MD5Hasher()
    self._AssertTestPathBinaryDigestMatch(
        hasher, [u'ímynd.dd'], expected_md5.decode(u'hex'))


if __name__ == '__main__':
  unittest.main()
